package DE_EPAGES::MultiStoreConnector::t::ProductSynchronization;
use base DE_EPAGES::Database::API::Test;

use strict;
use Test::More;

use DE_EPAGES::MultiStoreConnector::API::ProductSynchronization;
use DE_EPAGES::MultiStoreConnector::API::Test::TestHooksProduct qw ( GetCountersProduct ResetCountersProduct );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadObjectByGUID LoadObjectByPath ExistsObjectByPath );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::Trigger::API::XML::HookDeleteHandler qw (DeleteHooks);
use DE_EPAGES::Trigger::API::XML::HookImportHandler qw (ImportHooks);
use DE_EPAGES::Core::API::Error qw( GetError ExistsError);
#use DE_EPAGES::Database::API::Connection qw (GetCurrentDBHandle);

#global variables
our $TestShopAlias = 'MSIntegrationTestShop';
our $TestShopPath = '/Shops/' . 'MSIntegrationTestShop';
our $TestShopXML = 'MSIntegrationTestShop.xml';
our $HookXML = 'HooksProduct.xml';

our $hTestProductSourceArrayPosition = {
    'MSIntegrationTestProduct-01' => 0,
    'MSIntegrationTestProduct-02' => 1,
    'MSIntegrationInjectioTestProduct-01' => 0,
    'MSIntegrationNullValueTestProduct-01' => 0,
    'MSIntegrationNullValueTestProduct-02' => 1,
    'MSIntegrationDeleteTestProduct-01' => 0,
    'MSIntegrationDeleteTestProduct-02' => 1,
    'MSIntegrationPutTestProduct-01' => 0,
    'MSIntegrationPutTestProduct-02' => 1,
    'MSIntegrationTestProductTypeAssignment-01' => 0,
    'MSIntegrationTestProductTypeAssignment-02' => 1,
};

sub test_parameter_type {
    
    #get variables
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductSynchronization->new();
    
    #info of attributes and their expected data types
    my $hAttributeParameters = {
        'alias' => 'string',
        'position' => 'integer',
        'isvisible' => 'boolean',
        'visibleinnavigation' => 'boolean',
        'imagesmall' => 'string',
        'imagemedium' => 'string',
        'taxclass' => 'string',
        'isnew' => 'boolean',
        'pricequantity' => 'number',
        'minorder' => 'number',
        'intervalorder' => 'number',
        'weight' => 'number',
        'refunit' => 'string',
        'refamount' => 'integer',
        'refcontentamount' => 'number',
        'stocklevel' => 'number',
        'stocklevelalert' => 'number',
        #'manufacturer' => 'string',
        #'manufacturersku' => 'string',
        'deliveryperiod' => 'string',
        'length' => 'number',
        'height' => 'number',
        'width' => 'number',
        'isavailable' => 'boolean',
        #'availabilitydate' => 'string',
        'imagemediumsmall' => 'string',
        'imagemediumlarge' => 'string',
        'imagelarge' => 'string',
        'imagehotdeal' => 'string',
        #'superproduct' => 'string',
        'hassubownprices' => 'boolean',
        #'defaultsubproduct' => 'string',
        #'newnessdate' => 'datetime',
        'isappointment' => 'boolean',
        'canpayappointment' => 'boolean',
        'durationamount' => 'integer',
        'durationunit' => 'string',
        'ratingaverage' => 'number',
        'prepaymenttype' => 'string',
        'prepaymentvalue' => 'number',
        'isdownloadproduct' => 'boolean',
        'iscustomizable' => 'boolean',
        #'expirydate' => 'datetime'},
        'ismaster' => 'boolean',
        'upcean' => 'string',
        'isdailyprice' => 'boolean',
        'weightunit' => 'string',
        'orderunit' => 'string',
        'defaultsubproduct' => 'string'
    };

    foreach my $AttributeName (keys %$hAttributeParameters){
        is($Synchronizer->getAttributeParameterType($AttributeName), $hAttributeParameters->{$AttributeName} , 'parameter type attribute \'' . $AttributeName . '\' correct')
    }
    
}

sub test_mandatory {

    #get variables
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductSynchronization->new();

    #info of attributes mandatory
    my $hAttributeMandatory = {
        'alias' => 1,
        'position' => 1,
        'isvisible' => 1,
        'visibleinnavigation' => 1,
        'imagesmall' => 0,
        'imagemedium' => 0,
        'taxclass' => 0,
        'isnew' => 1,
        'pricequantity' => 0,
        'minorder' => 0,
        'intervalorder' => 0,
        'weight' => 0,
        'refunit' => 0,
        'refamount' => 0,
        'refcontentamount' => 0,
        'stocklevel' => 0,
        'stocklevelalert' => 0,
        #'manufacturer' => 0,
        #'manufacturersku' => 0,
        'deliveryperiod' => 0,
        'length' => 0,
        'height' => 0,
        'width' => 0,
        'isavailable' => 1,
        #'availabilitydate' => 0,
        'imagemediumsmall' => 0,
        'imagemediumlarge' => 0,
        'imagelarge' => 0,
        'imagehotdeal' => 0,
        #'superproduct' => 0,
        'hassubownprices' => 1,
        #'defaultsubproduct' => 0,
        #'newnessdate' => 0,
        'isappointment' => 1,
        'canpayappointment' => 1,
        'durationamount' => 0,
        'durationunit' => 0,
        'ratingaverage' => 0,
        'prepaymenttype' => 0,
        'prepaymentvalue' => 0,
        'isdownloadproduct' => 1,
        'iscustomizable' => 1,
        #'expirydate' => 0,
        'ismaster' => 1,
        'upcean' => 0,
        'isdailyprice' => 1,
        'weightunit' => 0,
        'orderunit' => 0,
        'defaultsubproduct' => 0
    };
    
    foreach my $AttributeName (keys %$hAttributeMandatory){
        is($Synchronizer->getAttributeMandatory($AttributeName), $hAttributeMandatory->{$AttributeName} , 'mandatory attribute \'' . $AttributeName . '\' correct')
    }
}

sub test_products{
    #get variables
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductSynchronization->new();
    
    # check for empty products
    my $aAllProducts = $Synchronizer->getProductsContentByShopAlias($TestShopAlias);
    
    ok(scalar(@$aAllProducts) == 0, 'shop has no products');
    
    #get test data
    my $aTestProducts = [{
        'alias' => 'MSIntegrationTestProduct-01',
        'product_type' => 'Jacket',
        'position' => 17,
        'isvisible' => 'true',
        'visibleinnavigation' => 'true',
        'imagesmall' => 'YD05F_s.JPG',
        'imagemedium' => 'YD05F_m.JPG',
        'taxclass' => 'normal',
        'isnew' => 'true',
        'isdailyprice' => 'false',
        'orderunit' => 'piece',
        'pricequantity' => 1.0,
        'minorder' => 1.0,
        'intervalorder' => 1.0,
        'weightunit' => 'kilogram',
        'weight' => 1.17,
        'refunit' => 'piece',
        'refamount' => 1,
        'refcontentamount' => 1.0,
        'stocklevel' => 3.0,
        'stocklevelalert' => 1.0,
        #'manufacturer' => 'NGK', needs manufacturer synchronization first
        #'manufacturersku' => ???,  needs manufacturer synchronization first
        'deliveryperiod' => 'test',
        'length' => 2.0,
        'height' => 3.0,
        'width' => 4.0,
        'isavailable' => 'true',
        #'availabilitydate' => ???,
        'imagemediumsmall' => 'YD05F_ms.JPG',
        'imagemediumlarge' => 'YD05F_ml.JPG',
        'imagelarge' => 'YD05F.JPG',
        'imagehotdeal' => 'YD05F_h.JPG',
        #'superproductid' => '',
        'defaultsubproduct' => '',
        'upcean' => "0087295184547",
        #'newnessdate' => ???,
        'isappointment' => 'false',
        'canpayappointment' => 'false',
        'durationamount' => 17,
        'durationunit' => 'minute',
        'ratingaverage' => 3.5,
        'prepaymenttype' => 'test_01',
        'prepaymentvalue' => 17.0,
        #'ecoparticipationcategoryid' => ???,
        'isdownloadproduct' => 'false',
        'iscustomizable' => 'false',
        #'expirydate' => ???
        'ismaster' => 'false',
        'hassubownprices' => 'true',
    },
    {
        'alias' => 'MSIntegrationTestProduct-02',
        'product_type' => 'Jacket',
        'position' => 0,
        'isvisible' => 'false',
        'visibleinnavigation' => 'false',
        'imagesmall' => '',
        'imagemedium' => '',
        'taxclass' => 'normal',
        'isnew' => 'false',
        'isdailyprice' => 'true',
        'orderunit' => 'piece',
        'pricequantity' => 1.0,
        'minorder' => 1.0,
        'intervalorder' => 1.0,
        'weightunit' => 'kilogram',
        'weight' => 999999.17,
        'refunit' => 'piece',
        'refamount' => 1,
        'refcontentamount' => 1.0,
        'stocklevel' => 999999.0,
        'stocklevelalert' => 17.0,
        #'manufacturer' => 'NGK', needs manufacturer synchronization first
        #'manufacturersku' => ???,  needs manufacturer synchronization first
        'deliveryperiod' => 'test2',
        'length' => 90.0,
        'height' => 110.0,
        'width' => 20000.0,
        'isavailable' => 'true',
        #'availabilitydate' => ???,
        'imagemediumsmall' => '',
        'imagemediumlarge' => '',
        'imagelarge' => '',
        'imagehotdeal' => '',
        #'superproduct' => '',
        'defaultsubproduct' => '',
        'upcean' => "",
        #'newnessdate' => ???,
        'isappointment' => 'true',
        'canpayappointment' => 'true',
        'durationamount' => 0,
        'durationunit' => 'minute',
        'ratingaverage' => 0.5,
        'prepaymenttype' => '',
        'prepaymentvalue' => 0.0,
        #'ecoparticipationcategoryid' => ???,
        'isdownloadproduct' => 'true',
        'iscustomizable' => 'true',
        #'expirydate' => ???
        'ismaster' => 'true',
        'hassubownprices' => 'false',
    }
    ];
    
    #####TEST PRODUCTS#####
    foreach my $hProduct (@$aTestProducts){
    
        #check if test products not exists
        ok(not (ExistsObjectByPath($TestShopPath . '/Products/' . $hProduct->{'alias'})), 'test product "' . $hProduct->{'alias'} . '" not exists');
    }
    #add products
    my $aProductsResult = $Synchronizer->addProductsToShop($TestShopAlias, $aTestProducts);
    my $hTestProductReturnArrayPosition;
    my $aReturnGUIDs;
    my $Counter = -1;
    
    #remember positions and guids
    foreach my $hResultProduct (@$aProductsResult){
        #increase counter
        $Counter += 1;
        
        #get info
        my $ResultAlias = $hResultProduct->{'alias'};
        my $ResultGUID = $hResultProduct->{'guid'};
        
        #remember position
        $hTestProductReturnArrayPosition->{$ResultAlias} = $Counter;
        
        #remember guids
        push(@$aReturnGUIDs, $ResultGUID);
        
    }
    
    #start testing
    foreach my $hProduct (@$aTestProducts){
    
        #get positions
        my $ResultPosition = $hTestProductReturnArrayPosition->{$hProduct->{'alias'}};
        my $SourcePosition = $hTestProductSourceArrayPosition->{$hProduct->{'alias'}};
    
        #get result info
        my $ResultGUID = $aProductsResult->[$ResultPosition]->{'guid'};
        my $ResultAlias = $aProductsResult->[$ResultPosition]->{'alias'};                                
        my $ResultEP6ID = $aProductsResult->[$ResultPosition]->{'epages6_objectid'};
        
        ok(((defined $ResultGUID) and ($ResultGUID ne '')),'guid exists and is not undef');
        is($ResultAlias, $hProduct->{'alias'},'alias "' . $hProduct->{'alias'} . '" is correct');
        ok(((defined $ResultEP6ID) and ($ResultEP6ID ne '')),'epages6 id exists and is not undef');
        
        #get object from database
        my $AddedTestObject = LoadObject($ResultEP6ID);
        my $AddedTestObjectByGUID = LoadObjectByGUID($ResultGUID);
        
        #check content with epages6 api
        is($AddedTestObject->id(), $AddedTestObjectByGUID->id(),"GUID is correct, epages 6 object api");
        is($AddedTestObject->get('Position'), $aTestProducts->[$SourcePosition]->{'position'}, 'position from db is correct, epages 6 object api');
        is($AddedTestObject->alias(), $aTestProducts->[$SourcePosition]->{'alias'}, 'alias from db is correct, epages 6 object api');
        is($AddedTestObject->get('IsVisible'), $aTestProducts->[$SourcePosition]->{'isvisible'}, 'is visible from db is correct, epages 6 object api');
        is($AddedTestObject->get('VisibleInNavigation'), $aTestProducts->[$SourcePosition]->{'visibleinnavigation'}, 'visible in navigation from db is correct, epages 6 object api');
        is($AddedTestObject->get('ImageSmall'), $aTestProducts->[$SourcePosition]->{'imagesmall'}, 'imagesmall from db is correct, epages 6 object api');
        is($AddedTestObject->get('ImageMedium'), $aTestProducts->[$SourcePosition]->{'imagemedium'}, 'imagemedium from db is correct, epages 6 object api');
        is($AddedTestObject->get('TaxClass')->alias, $aTestProducts->[$SourcePosition]->{'taxclass'}, 'taxclass from db is correct, epages 6 object api');
        is($AddedTestObject->get('IsNew'), $aTestProducts->[$SourcePosition]->{'isnew'}, 'isnew from db is correct, epages 6 object api');
        is($AddedTestObject->get('IsDailyPrice'), $aTestProducts->[$SourcePosition]->{'isdailyprice'}, 'daily price from db is correct, epages 6 object api');
        is($AddedTestObject->get('OrderUnit')->alias, $aTestProducts->[$SourcePosition]->{'orderunit'}, 'order unit from db is correct, epages 6 object api');
        is($AddedTestObject->get('PriceQuantity'), $aTestProducts->[$SourcePosition]->{'pricequantity'}, 'price quantity from db is correct, epages 6 object api');
        is($AddedTestObject->get('MinOrder'), $aTestProducts->[$SourcePosition]->{'minorder'}, 'minimal order from db is correct, epages 6 object api');
        is($AddedTestObject->get('IntervalOrder'), $aTestProducts->[$SourcePosition]->{'intervalorder'}, 'interval order from db is correct, epages 6 object api');
        is($AddedTestObject->get('WeightUnit')->alias, $aTestProducts->[$SourcePosition]->{'weightunit'}, 'weight unit from db is correct, epages 6 object api');
        is($AddedTestObject->get('Weight'), $aTestProducts->[$SourcePosition]->{'weight'}, 'weight from db is correct, epages 6 object api');
        is($AddedTestObject->get('RefUnit')->alias, $aTestProducts->[$SourcePosition]->{'refunit'}, 'referential unit from db is correct, epages 6 object api');
        is($AddedTestObject->get('RefAmount'), $aTestProducts->[$SourcePosition]->{'refamount'}, 'referential amount from db is correct, epages 6 object api');
        is($AddedTestObject->get('RefContentAmount'), $aTestProducts->[$SourcePosition]->{'refcontentamount'}, 'referential content amount from db is correct, epages 6 object api');
        is($AddedTestObject->get('StockLevel'), $aTestProducts->[$SourcePosition]->{'stocklevel'}, 'stocklevel from db is correct, epages 6 object api');
        is($AddedTestObject->get('StockLevelAlert'), $aTestProducts->[$SourcePosition]->{'stocklevelalert'}, 'stocklevel alert from db is correct, epages 6 object api');
        is($AddedTestObject->get('Length'), $aTestProducts->[$SourcePosition]->{'length'}, 'length from db is correct, epages 6 object api');
        is($AddedTestObject->get('Height'), $aTestProducts->[$SourcePosition]->{'height'}, 'height from db is correct, epages 6 object api');
        is($AddedTestObject->get('Width'), $aTestProducts->[$SourcePosition]->{'width'}, 'width from db is correct, epages 6 object api');
        is($AddedTestObject->get('IsAvailable'), $aTestProducts->[$SourcePosition]->{'isavailable'}, 'is available from db is correct, epages 6 object api');
        is($AddedTestObject->get('ImageMediumSmall'), $aTestProducts->[$SourcePosition]->{'imagemediumsmall'}, 'imagemediumsmall from db is correct, epages 6 object api');
        is($AddedTestObject->get('ImageMediumLarge'), $aTestProducts->[$SourcePosition]->{'imagemediumlarge'}, 'imagemediumlarge from db is correct, epages 6 object api');
        is($AddedTestObject->get('ImageLarge'), $aTestProducts->[$SourcePosition]->{'imagelarge'}, 'imagelarge from db is correct, epages 6 object api');
        is($AddedTestObject->get('ImageHotDeal'), $aTestProducts->[$SourcePosition]->{'imagehotdeal'}, 'imagehotdeal from db is correct, epages 6 object api');
        ok($AddedTestObject->get('UPCEAN') eq $aTestProducts->[$SourcePosition]->{'upcean'}, 'upcean order from db is correct, epages 6 object api');
        is($AddedTestObject->get('IsAppointment'), $aTestProducts->[$SourcePosition]->{'isappointment'}, 'isappointment from db is correct, epages 6 object api');
        is($AddedTestObject->get('CanPayAppointment'), $aTestProducts->[$SourcePosition]->{'canpayappointment'}, 'can pay appointment from db is correct, epages 6 object api');
        is($AddedTestObject->get('DurationAmount'), $aTestProducts->[$SourcePosition]->{'durationamount'}, 'duration amount from db is correct, epages 6 object api');
        is($AddedTestObject->get('DurationUnit')->alias, $aTestProducts->[$SourcePosition]->{'durationunit'}, 'duration unit from db is correct, epages 6 object api');
        is($AddedTestObject->get('RatingAverage'), $aTestProducts->[$SourcePosition]->{'ratingaverage'}, 'rating average from db is correct, epages 6 object api');
        ok($AddedTestObject->get('PrepaymentType') eq $aTestProducts->[$SourcePosition]->{'prepaymenttype'}, 'pre payment type from db is correct, epages 6 object api');
        ok($AddedTestObject->get('PrepaymentValue') eq $aTestProducts->[$SourcePosition]->{'prepaymentvalue'}, 'pre payment value from db is correct, epages 6 object api');
        is($AddedTestObject->get('IsDownloadProduct'), $aTestProducts->[$SourcePosition]->{'isdownloadproduct'}, 'isdownloadproduct from db is correct, epages 6 object api');
        is($AddedTestObject->get('IsCustomizable'), $aTestProducts->[$SourcePosition]->{'iscustomizable'}, 'iscustomizable from db is correct, epages 6 object api');
        is($AddedTestObject->get('IsMaster'), $aTestProducts->[$SourcePosition]->{'ismaster'}, 'ismaster from db is correct, epages 6 object api');

        # verify that hooks were triggered
        my $HookCounters = GetCountersProduct();
        is($HookCounters->{'OBJ_InsertProduct'}->{$ResultEP6ID}, 1, 'Hook OBJ_InsertProduct triggered correctly');
        is($HookCounters->{'OBJ_InsertContentObject'}->{$ResultEP6ID}, 1, 'Hook OBJ_InsertContentObject triggered correctly');
        is($HookCounters->{'OBJ_InsertObject'}->{$ResultEP6ID}, 1, 'Hook OBJ_InsertObject triggered correctly');
    }
    
    #get results from synch api
    my $aPerformanceGetResult = $Synchronizer->getProductsContentByGUIDs($aReturnGUIDs);
    
    foreach my $hPerformanceGetResultForGUID ( @$aPerformanceGetResult){
    
        #get source positon
        my $SourcePosition = $hTestProductSourceArrayPosition->{$hPerformanceGetResultForGUID->{'alias'}};
        
        #check content
        is($hPerformanceGetResultForGUID->{'alias'}, $aTestProducts->[$SourcePosition]->{'alias'}, 'alias from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'position'}, $aTestProducts->[$SourcePosition]->{'position'}, 'position from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'isvisible'}, $aTestProducts->[$SourcePosition]->{'isvisible'}, 'is visible from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'visibleinnavigation'}, $aTestProducts->[$SourcePosition]->{'visibleinnavigation'}, 'visible in navigation from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'imagesmall'}, $aTestProducts->[$SourcePosition]->{'imagesmall'}, 'imagesmall from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'imagemedium'}, $aTestProducts->[$SourcePosition]->{'imagemedium'}, 'imagemedium from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'taxclass'}, $aTestProducts->[$SourcePosition]->{'taxclass'}, 'taxclass from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'isnew'}, $aTestProducts->[$SourcePosition]->{'isnew'}, 'isnew from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'isdailyprice'}, $aTestProducts->[$SourcePosition]->{'isdailyprice'}, 'daily price from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'orderunit'}, $aTestProducts->[$SourcePosition]->{'orderunit'}, 'order unit from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'pricequantity'}, $aTestProducts->[$SourcePosition]->{'pricequantity'}, 'price quantity from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'minorder'}, $aTestProducts->[$SourcePosition]->{'minorder'}, 'minimal order from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'intervalorder'}, $aTestProducts->[$SourcePosition]->{'intervalorder'}, 'interval order from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'weightunit'}, $aTestProducts->[$SourcePosition]->{'weightunit'}, 'weight unit from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'weight'}, $aTestProducts->[$SourcePosition]->{'weight'}, 'weight from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'refunit'}, $aTestProducts->[$SourcePosition]->{'refunit'}, 'referential unit from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'refamount'}, $aTestProducts->[$SourcePosition]->{'refamount'},'referential amount from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'refcontentamount'}, $aTestProducts->[$SourcePosition]->{'refcontentamount'}, 'referential content amount from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'stocklevel'}, $aTestProducts->[$SourcePosition]->{'stocklevel'}, 'stocklevel from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'stocklevelalert'}, $aTestProducts->[$SourcePosition]->{'stocklevelalert'}, 'stocklevel alert from db is correct, epages 6 connector synch api');
        #is($hPerformanceGetResultForGUID->{'manufacturer'}, $aTestProducts->[$SourcePosition]->{'manufacturer'}, needs manufacturer synchronization first
        #is($hPerformanceGetResultForGUID->{'manufacturersku'}, $aTestProducts->[$SourcePosition]->{'manufacturersku'}???,  needs manufacturer synchronization first
        #is($hPerformanceGetResultForGUID->{'deliveryperiod'}, $aTestProducts->[$SourcePosition]->{'deliveryperiod'}???,
        is($hPerformanceGetResultForGUID->{'length'}, $aTestProducts->[$SourcePosition]->{'length'}, 'length from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'height'}, $aTestProducts->[$SourcePosition]->{'height'}, 'height from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'width'}, $aTestProducts->[$SourcePosition]->{'width'}, 'width from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'isavailable'}, $aTestProducts->[$SourcePosition]->{'isavailable'}, 'is available from db is correct, epages 6 connector synch api');
        #is($hPerformanceGetResultForGUID->{'availabilitydate'}, $aTestProducts->[$SourcePosition]->{'availabilitydate'}???,
        is($hPerformanceGetResultForGUID->{'imagemediumsmall'}, $aTestProducts->[$SourcePosition]->{'imagemediumsmall'}, 'imagemediumsmall from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'imagemediumlarge'}, $aTestProducts->[$SourcePosition]->{'imagemediumlarge'}, 'imagemediumlarge from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'imagelarge'}, $aTestProducts->[$SourcePosition]->{'imagelarge'}, 'imagelarge from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'imagehotdeal'}, $aTestProducts->[$SourcePosition]->{'imagehotdeal'}, 'imagehotdeal from db is correct, epages 6 connector synch api');
        #is($hPerformanceGetResultForGUID->{'superproductid'}, $aTestProducts->[$SourcePosition]->{'superproductid'}???,
        #is($hPerformanceGetResultForGUID->{'defaultsubproductid'}, $aTestProducts->[$SourcePosition]->{'defaultsubproductid'}???,
        ok($hPerformanceGetResultForGUID->{'upcean'} eq $aTestProducts->[$SourcePosition]->{'upcean'}, 'upcean order from db is correct, epages 6 connector synch api');
        #is($hPerformanceGetResultForGUID->{'newnessdate'}, $aTestProducts->[$SourcePosition]->{'newnessdate'} ???,
        is($hPerformanceGetResultForGUID->{'isappointment'}, $aTestProducts->[$SourcePosition]->{'isappointment'}, 'isappointment from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'canpayappointment'}, $aTestProducts->[$SourcePosition]->{'canpayappointment'}, 'can pay appointment from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'durationamount'}, $aTestProducts->[$SourcePosition]->{'durationamount'}, 'duration amount from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'durationunit'}, $aTestProducts->[$SourcePosition]->{'durationunit'}, 'duration unit from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'ratingaverage'}, $aTestProducts->[$SourcePosition]->{'ratingaverage'}, 'rating average from db is correct, epages 6 connector synch api');
        ok($hPerformanceGetResultForGUID->{'prepaymenttype'} eq $aTestProducts->[$SourcePosition]->{'prepaymenttype'}, 'pre payment type from db is correct, epages 6 connector synch api');
        ok($hPerformanceGetResultForGUID->{'prepaymentvalue'} eq $aTestProducts->[$SourcePosition]->{'prepaymentvalue'}, 'pre payment value from db is correct, epages 6 connector synch api');
        #is($hPerformanceGetResultForGUID->{'ecoparticipationcategoryid'}, $aTestProducts->[$SourcePosition]->{'ecoparticipationcategoryid'}???,
        is($hPerformanceGetResultForGUID->{'isdownloadproduct'}, $aTestProducts->[$SourcePosition]->{'isdownloadproduct'}, 'isdownloadproduct from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'iscustomizable'}, $aTestProducts->[$SourcePosition]->{'iscustomizable'}, 'iscustomizable from db is correct, epages 6 connector synch api');
        #is($hPerformanceGetResultForGUID->{'expirydate'}, $aTestProducts->[$SourcePosition]->{'expirydate'} ???
        is($hPerformanceGetResultForGUID->{'ismaster'}, $aTestProducts->[$SourcePosition]->{'ismaster'}, 'ismaster from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'hassubownprices'}, $aTestProducts->[$SourcePosition]->{'hassubownprices'}, 'hassubownprices from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'product_type'}, $aTestProducts->[$SourcePosition]->{'product_type'}, 'product_type from db is correct, epages 6 connector synch api');
        
        #verify parameters
        ok($Synchronizer->isString($hPerformanceGetResultForGUID->{'alias'}), 'alias from db is string');
        ok($Synchronizer->isInteger($hPerformanceGetResultForGUID->{'position'}), 'position from db is integer');
        ok($Synchronizer->isBool($hPerformanceGetResultForGUID->{'isvisible'}), 'is visible from db is bool');
        ok($Synchronizer->isBool($hPerformanceGetResultForGUID->{'visibleinnavigation'}), 'visible in navigation from db is bool');
        ok($Synchronizer->isString($hPerformanceGetResultForGUID->{'imagesmall'}), 'imagesmall from db is string');
        ok($Synchronizer->isString($hPerformanceGetResultForGUID->{'imagemedium'}), 'imagemedium from db is string');
        ok($Synchronizer->isString($hPerformanceGetResultForGUID->{'taxclass'}), 'taxclass from db is string');
        ok($Synchronizer->isBool($hPerformanceGetResultForGUID->{'isnew'}), 'isnew from db is bool');
        ok($Synchronizer->isBool($hPerformanceGetResultForGUID->{'isdailyprice'}), 'daily price from db is bool');
        ok($Synchronizer->isString($hPerformanceGetResultForGUID->{'orderunit'}), 'order unit from db is string');
        ok($Synchronizer->isNumber($hPerformanceGetResultForGUID->{'pricequantity'}), 'price quantity from db is number');
        ok($Synchronizer->isNumber($hPerformanceGetResultForGUID->{'minorder'}), 'minimal order from db is number');
        ok($Synchronizer->isNumber($hPerformanceGetResultForGUID->{'intervalorder'}), 'interval order from db is number');
        ok($Synchronizer->isString($hPerformanceGetResultForGUID->{'weightunit'}), 'weight unit from db is string');
        ok($Synchronizer->isNumber($hPerformanceGetResultForGUID->{'weight'}), 'weight from db is number');
        ok($Synchronizer->isString($hPerformanceGetResultForGUID->{'refunit'}), 'referential unit from db is string');
        ok($Synchronizer->isInteger($hPerformanceGetResultForGUID->{'refamount'}),'referential amount from db is integer');
        ok($Synchronizer->isNumber($hPerformanceGetResultForGUID->{'refcontentamount'}), 'referential content amount from db is number');
        ok($Synchronizer->isNumber($hPerformanceGetResultForGUID->{'stocklevel'}), 'stocklevel from db is number');
        ok($Synchronizer->isNumber($hPerformanceGetResultForGUID->{'stocklevelalert'}), 'stocklevel alert from db is number');
        #ok($Synchronizer->isString($hPerformanceGetResultForGUID->{'manufacturer'}), 'stocklevel alert from db is string');  -- needs manufacturer synchronization first
        #ok($Synchronizer->isString($hPerformanceGetResultForGUID->{'manufacturersku'}, 'stocklevel alert from db is string');  -- needs manufacturer synchronization first
        ok($Synchronizer->isString($hPerformanceGetResultForGUID->{'deliveryperiod'}), 'deliveryperiod from db is string');
        ok($Synchronizer->isNumber($hPerformanceGetResultForGUID->{'length'}), 'length from db is number');
        ok($Synchronizer->isNumber($hPerformanceGetResultForGUID->{'height'}), 'height from db is number');
        ok($Synchronizer->isNumber($hPerformanceGetResultForGUID->{'width'}), 'width from db is number');
        ok($Synchronizer->isBool($hPerformanceGetResultForGUID->{'isavailable'}), 'is available from db is bool');
        #ok($Synchronizer->isDateTime($hPerformanceGetResultForGUID->{'availabilitydate'}), 'is available from db is date time');
        ok($Synchronizer->isString($hPerformanceGetResultForGUID->{'imagemediumsmall'}), 'imagemediumsmall from db is string');
        ok($Synchronizer->isString($hPerformanceGetResultForGUID->{'imagemediumlarge'}), 'imagemediumlarge from db is string');
        ok($Synchronizer->isString($hPerformanceGetResultForGUID->{'imagelarge'}), 'imagelarge from db is string');
        ok($Synchronizer->isString($hPerformanceGetResultForGUID->{'imagehotdeal'}), 'imagehotdeal from db is string');
        #is($hPerformanceGetResultForGUID->{'superproductid'}, $aTestProducts->[$SourcePosition]->{'superproductid'}???,
        #is($hPerformanceGetResultForGUID->{'defaultsubproductid'}, $aTestProducts->[$SourcePosition]->{'defaultsubproductid'}???,
        ok($Synchronizer->isString($hPerformanceGetResultForGUID->{'upcean'}), 'upcean order from db is string');
        #is($hPerformanceGetResultForGUID->{'newnessdate'}, $aTestProducts->[$SourcePosition]->{'newnessdate'} ???,
        ok($Synchronizer->isBool($hPerformanceGetResultForGUID->{'isappointment'}), 'isappointment from db is boolean');
        ok($Synchronizer->isBool($hPerformanceGetResultForGUID->{'canpayappointment'}), 'can pay appointment from db is boolean');
        ok($Synchronizer->isInteger($hPerformanceGetResultForGUID->{'durationamount'}), 'duration amount from db is integer');
        ok($Synchronizer->isString($hPerformanceGetResultForGUID->{'durationunit'}), 'duration unit from db is string');
        ok($Synchronizer->isNumber($hPerformanceGetResultForGUID->{'ratingaverage'}), 'rating average from db is number');
        ok($Synchronizer->isString($hPerformanceGetResultForGUID->{'prepaymenttype'}), 'pre payment type from db is string');
        ok($Synchronizer->isNumber($hPerformanceGetResultForGUID->{'prepaymentvalue'}), 'pre payment value from db is number');
        #is($hPerformanceGetResultForGUID->{'ecoparticipationcategoryid'}, $aTestProducts->[$SourcePosition]->{'ecoparticipationcategoryid'}???,
        ok($Synchronizer->isBool($hPerformanceGetResultForGUID->{'isdownloadproduct'}), 'isdownloadproduct from db is boolean');
        ok($Synchronizer->isBool($hPerformanceGetResultForGUID->{'iscustomizable'}), 'iscustomizable from db is boolean');
        #ok($Synchronizer->isDateTime($hPerformanceGetResultForGUID->{'expirydate'}), 'expirydate from db is date time');
        ok($Synchronizer->isBool($hPerformanceGetResultForGUID->{'ismaster'}), 'ismaster from db is boolean');
        ok($Synchronizer->isBool($hPerformanceGetResultForGUID->{'hassubownprices'}), 'hassubownprices from db is boolean');
        ok($Synchronizer->isString($hPerformanceGetResultForGUID->{'product_type'}), 'product_type from db is string');
    }
    
    # delete products
    delete_all_products();
    
    
    #get results from synch api
    $aAllProducts = $Synchronizer->getProductsContentByShopAlias($TestShopAlias);
    
    ok(scalar(@$aAllProducts) == 0, 'shop has no products');
}

#test post products focus on product types assignment
sub test_post_products_product_type_assignment{
    
    #get variables
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductSynchronization->new();
    
    #get results from synch api
    my $aAllProducts = $Synchronizer->getProductsContentByShopAlias($TestShopAlias);
    
    ok(scalar(@$aAllProducts) == 0, 'shop has no products');
    
    #get test data
    my $aTestPositiveProducts = [{
        'alias' => 'MSIntegrationTestProductTypeAssignment-01',
        'product_type' => 'Jacket',
        'position' => 17,
        'isvisible' => 'true',
        'visibleinnavigation' => 'true',
        'imagesmall' => 'YD05F_s.JPG',
        'imagemedium' => 'YD05F_m.JPG',
        'taxclass' => 'normal',
        'isnew' => 'true',
        'isdailyprice' => 'false',
        'orderunit' => 'piece',
        'pricequantity' => 1.0,
        'minorder' => 1.0,
        'intervalorder' => 1.0,
        'weightunit' => 'kilogram',
        'weight' => 1.17,
        'refunit' => 'piece',
        'refamount' => 1,
        'refcontentamount' => 1.0,
        'stocklevel' => 3.0,
        'stocklevelalert' => 1.0,
        #'manufacturer' => 'NGK', needs manufacturer synchronization first
        #'manufacturersku' => ???,  needs manufacturer synchronization first
        'deliveryperiod' => 'test',
        'length' => 2.0,
        'height' => 3.0,
        'width' => 4.0,
        'isavailable' => 'true',
        #'availabilitydate' => ???,
        'imagemediumsmall' => 'YD05F_ms.JPG',
        'imagemediumlarge' => 'YD05F_ml.JPG',
        'imagelarge' => 'YD05F.JPG',
        'imagehotdeal' => 'YD05F_h.JPG',
        #'superproductid' => '',
        'defaultsubproduct' => '',
        'upcean' => "0087295184547",
        #'newnessdate' => ???,
        'isappointment' => 'false',
        'canpayappointment' => 'false',
        'durationamount' => 17,
        'durationunit' => 'minute',
        'ratingaverage' => 3.5,
        'prepaymenttype' => 'test_01',
        'prepaymentvalue' => 17.0,
        #'ecoparticipationcategoryid' => ???,
        'isdownloadproduct' => 'false',
        'iscustomizable' => 'false',
        #'expirydate' => ???
        'ismaster' => 'false',
        'hassubownprices' => 'true',
    },
    {
        'alias' => 'MSIntegrationTestProductTypeAssignment-02',
        'product_type' => 'Shoe',
        'position' => 0,
        'isvisible' => 'false',
        'visibleinnavigation' => 'false',
        'imagesmall' => '',
        'imagemedium' => '',
        'taxclass' => 'normal',
        'isnew' => 'false',
        'isdailyprice' => 'true',
        'orderunit' => 'piece',
        'pricequantity' => 1.0,
        'minorder' => 1.0,
        'intervalorder' => 1.0,
        'weightunit' => 'kilogram',
        'weight' => 999999.17,
        'refunit' => 'piece',
        'refamount' => 1,
        'refcontentamount' => 1.0,
        'stocklevel' => 999999.0,
        'stocklevelalert' => 17.0,
        #'manufacturer' => 'NGK', needs manufacturer synchronization first
        #'manufacturersku' => ???,  needs manufacturer synchronization first
        'deliveryperiod' => 'test2',
        'length' => 90.0,
        'height' => 110.0,
        'width' => 20000.0,
        'isavailable' => 'true',
        #'availabilitydate' => ???,
        'imagemediumsmall' => '',
        'imagemediumlarge' => '',
        'imagelarge' => '',
        'imagehotdeal' => '',
        #'superproduct' => '',
        'defaultsubproduct' => '',
        'upcean' => "",
        #'newnessdate' => ???,
        'isappointment' => 'true',
        'canpayappointment' => 'true',
        'durationamount' => 0,
        'durationunit' => 'minute',
        'ratingaverage' => 0.5,
        'prepaymenttype' => '',
        'prepaymentvalue' => 0.0,
        #'ecoparticipationcategoryid' => ???,
        'isdownloadproduct' => 'true',
        'iscustomizable' => 'true',
        #'expirydate' => ???
        'ismaster' => 'true',
        'hassubownprices' => 'false',
    }
    ];
    
    my $aTestNegativeProducts = [{
        'alias' => 'MSIntegrationTestFailProductTypeAssignment-01',
        #this is missing here test should fail ('product_type' => 'Jacket',)
        'position' => 17,
        'isvisible' => 'true',
        'visibleinnavigation' => 'true',
        'imagesmall' => 'YD05F_s.JPG',
        'imagemedium' => 'YD05F_m.JPG',
        'taxclass' => 'normal',
        'isnew' => 'true',
        'isdailyprice' => 'false',
        'orderunit' => 'piece',
        'pricequantity' => 1.0,
        'minorder' => 1.0,
        'intervalorder' => 1.0,
        'weightunit' => 'kilogram',
        'weight' => 1.17,
        'refunit' => 'piece',
        'refamount' => 1,
        'refcontentamount' => 1.0,
        'stocklevel' => 3.0,
        'stocklevelalert' => 1.0,
        #'manufacturer' => 'NGK', needs manufacturer synchronization first
        #'manufacturersku' => ???,  needs manufacturer synchronization first
        'deliveryperiod' => 'test',
        'length' => 2.0,
        'height' => 3.0,
        'width' => 4.0,
        'isavailable' => 'true',
        #'availabilitydate' => ???,
        'imagemediumsmall' => 'YD05F_ms.JPG',
        'imagemediumlarge' => 'YD05F_ml.JPG',
        'imagelarge' => 'YD05F.JPG',
        'imagehotdeal' => 'YD05F_h.JPG',
        #'superproductid' => '',
        'defaultsubproduct' => '',
        'upcean' => "0087295184547",
        #'newnessdate' => ???,
        'isappointment' => 'false',
        'canpayappointment' => 'false',
        'durationamount' => 17,
        'durationunit' => 'minute',
        'ratingaverage' => 3.5,
        'prepaymenttype' => 'test_01',
        'prepaymentvalue' => 17.0,
        #'ecoparticipationcategoryid' => ???,
        'isdownloadproduct' => 'false',
        'iscustomizable' => 'false',
        #'expirydate' => ???
        'ismaster' => 'false',
        'hassubownprices' => 'true',
    }];
    
    #####TEST PRODUCTS#####
    foreach my $hProduct (@$aTestPositiveProducts){
    
        #check if test products not exists
        ok(not (ExistsObjectByPath($TestShopPath . '/Products/' . $hProduct->{'alias'})), 'test product "' . $hProduct->{'alias'} . '" not exists');
    }
    #add products
    $Synchronizer->addProductsToShop($TestShopAlias, $aTestPositiveProducts);
    
    #test if products exist now
    foreach my $hProduct (@$aTestPositiveProducts){
        #check if test products exists
        ok(ExistsObjectByPath($TestShopPath . '/Products/' . $hProduct->{'alias'}), 'test product "' . $hProduct->{'alias'} . '" exists');
        
        #check if class name is correct
        my $Product = LoadObjectByPath($TestShopPath . '/Products/' . $hProduct->{'alias'});
        is($Product->get('Class')->alias,$hProduct->{'product_type'}, "correct product type"); # note dont use className here because it returns 'Product' instead of the ProductType class
    }
    
    #try fail test
    eval{
        #add products
        $Synchronizer->addProductsToShop($TestShopAlias, $aTestNegativeProducts);
    };
    if (ExistsError()){
        my $Error = GetError();
        is($Error->code, 'MissingMandatoryAttribute', 'post not possible without a product_type attribute');
    }
    else{
        ok(0,'post not possible without a product_type attribute');
    }
    
    # delete products
    delete_all_products();
    
    #get results from synch api
    $aAllProducts = $Synchronizer->getProductsContentByShopAlias($TestShopAlias);
    
    ok(scalar(@$aAllProducts) == 0, 'shop has no products');
}

sub test_sql_injection {

    #get variables
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductSynchronization->new();
    
    #get results from synch api
    my $aAllProducts = $Synchronizer->getProductsContentByShopAlias($TestShopAlias);
    
    ok(scalar(@$aAllProducts) == 0, 'shop has no products');
    
    #get test data
    my $SQLInjection = "SET foreign_key_checks = 0; DROP TABLE object;";
    my $aInjectionTestProducts = [{
        'alias' => 'MSIntegrationInjectionTestProduct-01',
        'product_type' => 'Jacket',
        'position' => 0,
        'isvisible' => 'false',
        'visibleinnavigation' => 'false',
        'imagesmall' => '',
        'imagemedium' => '',
        'taxclass' => 'normal',
        'isnew' => 'false',
        'isdailyprice' => 'true',
        'orderunit' => 'piece',
        'pricequantity' => 1.0,
        'minorder' => 1.0,
        'intervalorder' => 1.0,
        'weightunit' => 'kilogram',
        'weight' => 999999.17,
        'refunit' => 'piece',
        'refamount' => 1,
        'refcontentamount' => 1.0,
        'stocklevel' => 999999.0,
        'stocklevelalert' => 17.0,
        #'manufacturer' => 'NGK', needs manufacturer synchronization first
        #'manufacturersku' => ???,  needs manufacturer synchronization first
        'deliveryperiod' => 'test2',
        'length' => 90.0,
        'height' => 110.0,
        'width' => 20000.0,
        'isavailable' => 'true',
        #'availabilitydate' => ???,
        'imagemediumsmall' => $SQLInjection,
        'imagemediumlarge' => '',
        'imagelarge' => '',
        'imagehotdeal' => '',
        #'superproduct' => 'test',
        'defaultsubproduct' => 'test',
        'upcean' => "",
        #'newnessdate' => ???,
        'isappointment' => 'true',
        'canpayappointment' => 'true',
        'durationamount' => 0,
        'durationunit' => 'minute',
        'ratingaverage' => 0.5,
        'prepaymenttype' => '',
        'prepaymentvalue' => 0.0,
        #'ecoparticipationcategoryid' => ???,
        'isdownloadproduct' => 'true',
        'iscustomizable' => 'true',
        #'expirydate' => ???
        'ismaster' => 'true',
        'hassubownprices' => 'true',
    }];
    
    #test if injection products not exists
    foreach my $hInjectionProduct (@$aInjectionTestProducts){
    
        #check if fail test products not exists
        ok(not (ExistsObjectByPath($TestShopPath . '/Products/' . $hInjectionProduct->{'alias'})), 'test product "' . $hInjectionProduct->{'alias'} . '" not exists');
    }
    
    #add injection products
    my $aInjectionProductsResult = $Synchronizer->addProductsToShop($TestShopAlias, $aInjectionTestProducts);
    
    
    foreach my $hInjectionProduct (@$aInjectionProductsResult){
        
        #get positions
        my $ResultPosition = 0;
    
        #get result info                              
        my $ResultEP6ID = $aInjectionProductsResult->[$ResultPosition]->{'epages6_objectid'};
        
        my $AddedInjectionTestObject = LoadObject($ResultEP6ID);
        
        #check if injection test products exists
        is($AddedInjectionTestObject->alias , $hInjectionProduct->{'alias'}, 'injection test product "' . $hInjectionProduct->{'alias'} . '" exists, database survived');
    }
    
    #start testing
    foreach my $hInjectionProductResult (@$aInjectionProductsResult){
    
        #get results from synch api
        my $aInjectionPerformanceGetResult = $Synchronizer->getProductsContentByGUIDs([$hInjectionProductResult->{'guid'}]);
        
        foreach my $hInjectionPerformanceGetResultForGUID ( @$aInjectionPerformanceGetResult){
            #get info
            my $SourcePosition = $hTestProductSourceArrayPosition->{$hInjectionPerformanceGetResultForGUID->{'alias'}};
            
            #test
            is($hInjectionPerformanceGetResultForGUID->{'imagemediumsmall'}, $aInjectionTestProducts->[$SourcePosition]->{'imagemediumsmall'}, 'imagemediumsmall injection from db is correct and data base survived, epages 6 connector synch api');
        }
    }
    
    # delete products
    delete_all_products();
    
    #get results from synch api
    $aAllProducts = $Synchronizer->getProductsContentByShopAlias($TestShopAlias);
    
    ok(scalar(@$aAllProducts) == 0, 'shop has no products');
}

sub test_null_value_products{
    
    #get variables
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductSynchronization->new();
    
    #get results from synch api
    my $aAllProducts = $Synchronizer->getProductsContentByShopAlias($TestShopAlias);
    
    ok(scalar(@$aAllProducts) == 0, 'shop has no products');
    
    #get test data
    my $aNullValueTestProducts = [
        {
            'alias' => 'MSIntegrationNullValueTestProduct-01',
            'product_type' => 'Jacket',
            'isdownloadproduct' => 'true',
            'isnew' => 'false',
            'isdailyprice' => 'false',
            'isvisible' => 'true',
            'visibleinnavigation' => 'false',
            'position' => 0,
            'hassubownprices' => 'false',
            'iscustomizable' => 'false',
            'ismaster' => 'false',
            'isavailable' => 'true',
            'canpayappointment' => 'true',
            'isappointment' => 'false',
            'taxclass' => undef,
            'orderunit' => undef,
            'pricequantity' => undef,
            'minorder' => undef,
            'intervalorder' => undef,
            'weightunit' => undef,
            'weight' => undef,
            'refunit' => undef,
            'refamount' => undef,
            'refcontentamount' => undef,
            'stocklevel' => undef,
            'stocklevelalert' => undef,
            'length' => undef,
            'height' => undef,
            'width' => undef,
            'imagemediumsmall' => undef,
            'imagemediumlarge' => undef,
            'imagelarge' => undef,
            'imagesmall' => undef,
            'imagemedium' => undef,
            'imagehotdeal' => undef,
            'upcean' => undef,
            'deliveryperiod' => undef
        },
        {
            'alias' => 'MSIntegrationNullValueTestProduct-02',
            'product_type' => 'Jacket',
            'isdownloadproduct' => 'true',
            'isnew' => 'false',
            'isdailyprice' => 'false',
            'isvisible' => 'true',
            'visibleinnavigation' => 'true',
            'position' => 0,
            'hassubownprices' => 'true',
            'iscustomizable' => 'true',
            'ismaster' => 'true',
            'isavailable' => 'false',
            'canpayappointment' => 'false',
            'isappointment' => 'false',
        }
    ];
    
    #####TEST NULL VALUE PRODUCTS#####
    #placeholder
    my $hNullValueTestProductReturnArrayPosition;
    my $aNullValueProductsResult;
    my $aNullValueGUIDs;
    
    my $Counter = -1;
    foreach my $hNullValueProduct (@$aNullValueTestProducts){
    
        #check if test products not exists
        ok(not (ExistsObjectByPath($TestShopPath . '/Products/' . $hNullValueProduct->{'alias'})), 'test product "' . $hNullValueProduct->{'alias'} . '" not exists');
    }
    eval{
        #add products
        $aNullValueProductsResult = $Synchronizer->addProductsToShop($TestShopAlias, $aNullValueTestProducts);
    };
    
    ok(not (ExistsError()), 'insert null value products successfully');
    
    #remember positions
    foreach my $hNullValueResultProduct (@$aNullValueProductsResult){
        #increase counter
        $Counter += 1;
        
        #get info
        my $ResultAlias = $hNullValueResultProduct->{'alias'};
        
        #remember position
        $hNullValueTestProductReturnArrayPosition->{$ResultAlias} = $Counter;
    }
    
    #get guids from result
    foreach my $hNullValueProduct (@$aNullValueTestProducts){
    
        #get positions
        my $NullValueResultPosition = $hNullValueTestProductReturnArrayPosition->{$hNullValueProduct->{'alias'}};
        
        #get info
        my $NullValueResultGUID = $aNullValueProductsResult->[$NullValueResultPosition]->{'guid'};
        
        #remember guids
        push(@$aNullValueGUIDs, $NullValueResultGUID);
    }
    
    #get results from synch api
    my $aNullValuePerformanceGetResults = $Synchronizer->getProductsContentByGUIDs($aNullValueGUIDs);
    
    #start testing
    foreach my $aNullValuePerformanceGetResult (@$aNullValuePerformanceGetResults){
        
        #get source positon
        my $NullValueSourcePosition = $hTestProductSourceArrayPosition->{$aNullValuePerformanceGetResult->{'alias'}};
        
        is($aNullValuePerformanceGetResult->{'alias'}, $aNullValueTestProducts->[$NullValueSourcePosition]->{'alias'}, 'alias from db is correct, epages 6 connector synch api');
    }
    
    # delete products
    delete_all_products();
    
    #get results from synch api
    $aAllProducts = $Synchronizer->getProductsContentByShopAlias($TestShopAlias);
    
    ok(scalar(@$aAllProducts) == 0, 'shop has no products');
}

sub test_non_product_guids {
    
    #get variables
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductSynchronization->new();
    
    #test empty return value if guid is not product
    my $NotProductGUID = LoadObjectByPath('/')->guid();
    
    is($Synchronizer->getProductsContentByGUIDs([$NotProductGUID]), undef,'empty result for guid that is not a product');
}

sub test_delete_products{
    #get test data
    my $aDeleteTestProducts = [{
        'alias' => 'MSIntegrationDeleteTestProduct-01',
        'product_type' => 'Jacket',
        'position' => 17,
        'isvisible' => 'true',
        'visibleinnavigation' => 'true',
        'imagesmall' => 'YD05F_s.JPG',
        'imagemedium' => 'YD05F_m.JPG',
        'taxclass' => 'normal',
        'isnew' => 'true',
        'isdailyprice' => 'false',
        'orderunit' => 'piece',
        'pricequantity' => 1.0,
        'minorder' => 1.0,
        'intervalorder' => 1.0,
        'weightunit' => 'kilogram',
        'weight' => 1.17,
        'refunit' => 'piece',
        'refamount' => 1,
        'refcontentamount' => 1.0,
        'stocklevel' => 3.0,
        'stocklevelalert' => 1.0,
        #'manufacturer' => 'NGK', needs manufacturer synchronization first
        #'manufacturersku' => ???,  needs manufacturer synchronization first
        'deliveryperiod' => 'test',
        'length' => 2.0,
        'height' => 3.0,
        'width' => 4.0,
        'isavailable' => 'true',
        #'availabilitydate' => ???,
        'imagemediumsmall' => 'YD05F_ms.JPG',
        'imagemediumlarge' => 'YD05F_ml.JPG',
        'imagelarge' => 'YD05F.JPG',
        'imagehotdeal' => 'YD05F_h.JPG',
        #'superproductid' => '',
        'defaultsubproduct' => '',
        'upcean' => "0087295184547",
        #'newnessdate' => ???,
        'isappointment' => 'false',
        'canpayappointment' => 'false',
        'durationamount' => 17,
        'durationunit' => 'minute',
        'ratingaverage' => 3.5,
        'prepaymenttype' => 'test_01',
        'prepaymentvalue' => 17.0,
        #'ecoparticipationcategoryid' => ???,
        'isdownloadproduct' => 'false',
        'iscustomizable' => 'false',
        #'expirydate' => ???
        'ismaster' => 'false',
        'hassubownprices' => 'true',
    },
    {
        'alias' => 'MSIntegrationDeleteTestProduct-02',
        'product_type' => 'Jacket',
        'position' => 0,
        'isvisible' => 'false',
        'visibleinnavigation' => 'false',
        'imagesmall' => '',
        'imagemedium' => '',
        'taxclass' => 'normal',
        'isnew' => 'false',
        'isdailyprice' => 'true',
        'orderunit' => 'piece',
        'pricequantity' => 1.0,
        'minorder' => 1.0,
        'intervalorder' => 1.0,
        'weightunit' => 'kilogram',
        'weight' => 999999.17,
        'refunit' => 'piece',
        'refamount' => 1,
        'refcontentamount' => 1.0,
        'stocklevel' => 999999.0,
        'stocklevelalert' => 17.0,
        #'manufacturer' => 'NGK', needs manufacturer synchronization first
        #'manufacturersku' => ???,  needs manufacturer synchronization first
        'deliveryperiod' => 'test2',
        'length' => 90.0,
        'height' => 110.0,
        'width' => 20000.0,
        'isavailable' => 'true',
        #'availabilitydate' => ???,
        'imagemediumsmall' => '',
        'imagemediumlarge' => '',
        'imagelarge' => '',
        'imagehotdeal' => '',
        #'superproduct' => '',
        'defaultsubproduct' => '',
        'upcean' => "",
        #'newnessdate' => ???,
        'isappointment' => 'true',
        'canpayappointment' => 'true',
        'durationamount' => 0,
        'durationunit' => 'minute',
        'ratingaverage' => 0.5,
        'prepaymenttype' => '',
        'prepaymentvalue' => 0.0,
        #'ecoparticipationcategoryid' => ???,
        'isdownloadproduct' => 'true',
        'iscustomizable' => 'true',
        #'expirydate' => ???
        'ismaster' => 'true',
        'hassubownprices' => 'false',
    }];
    
    #get variables
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductSynchronization->new();
    
    # check for empty shop
    my $aAllProducts = $Synchronizer->getProductsContentByShopAlias($TestShopAlias);
    
    ok(scalar(@$aAllProducts) == 0, 'shop has no products');
    
    #####TEST DELETE PRODUCTS#####
    foreach my $hDeleteProduct (@$aDeleteTestProducts){
    
        #check if test products not exists
        ok(not (ExistsObjectByPath($TestShopPath . '/Products/' . $hDeleteProduct->{'alias'})), 'test product "' . $hDeleteProduct->{'alias'} . '" not exists');
    }
    #add products
    my $aProductsResult = $Synchronizer->addProductsToShop($TestShopAlias, $aDeleteTestProducts);
    my $hTestProductReturnArrayPosition;
    my $aReturnGUIDs;
    my $Counter = -1;
    
    #remember positions and guids
    foreach my $hResultProduct (@$aProductsResult){
        #increase counter
        $Counter += 1;
        
        #get info
        my $ResultAlias = $hResultProduct->{'alias'};
        my $ResultGUID = $hResultProduct->{'guid'};
        
        #remember position
        $hTestProductReturnArrayPosition->{$ResultAlias} = $Counter;
        
        #remember guids
        push(@$aReturnGUIDs, $ResultGUID);
        
    }
    
    #start testing
    foreach my $hDeleteProduct (@$aDeleteTestProducts){
    
        #get positions
        my $ResultPosition = $hTestProductReturnArrayPosition->{$hDeleteProduct->{'alias'}};
    
        #get result info
        my $ResultGUID = $aProductsResult->[$ResultPosition]->{'guid'};
        my $ResultAlias = $aProductsResult->[$ResultPosition]->{'alias'};                                
        my $ResultEP6ID = $aProductsResult->[$ResultPosition]->{'epages6_objectid'};
        
        ok(((defined $ResultGUID) and ($ResultGUID ne '')),'guid exists and is not undef');
        is($ResultAlias, $hDeleteProduct->{'alias'},'alias "' . $hDeleteProduct->{'alias'} . '" is correct');
        ok(((defined $ResultEP6ID) and ($ResultEP6ID ne '')),'epages6 id exists and is not undef');
        
        #get object from database
        my $AddedTestObject = LoadObject($ResultEP6ID);
        my $AddedTestObjectByGUID = LoadObjectByGUID($ResultGUID);
        
        #check content with epages6 api
        is($AddedTestObject->id(), $AddedTestObjectByGUID->id(),"GUID is correct, epages 6 object api");
    }
    
    #get results from synch api
    my $aPerformanceGetResult = $Synchronizer->getProductsContentByGUIDs($aReturnGUIDs);
    
    foreach my $hPerformanceGetResultForGUID ( @$aPerformanceGetResult){
    
        #get source positon
        my $SourcePosition = $hTestProductSourceArrayPosition->{$hPerformanceGetResultForGUID->{'alias'}};
        
        #check content
        is($hPerformanceGetResultForGUID->{'alias'}, $aDeleteTestProducts->[$SourcePosition]->{'alias'}, 'alias from db is correct, epages 6 connector synch api');
    }
    
    #delete result from synch api
    my $aPerformanceDeleteResult = $Synchronizer->deleteProductsByGUIDs($aReturnGUIDs);
    
    #check result
    foreach my $hDeleteProductResult (@$aPerformanceDeleteResult){
        is($hDeleteProductResult->{'status'}, 'success','delete product result for guid "' . $hDeleteProductResult->{'guid'} . '" is successful');
        is($Synchronizer->getProductsContentByGUIDs([$hDeleteProductResult->{'guid'}]), undef, 'product with guid "' . $hDeleteProductResult->{'guid'} . '" not exists in db, epages 6 connector synch api');
        eval{
            LoadObjectByGUID($hDeleteProductResult->{'guid'});
        };
        ok(ExistsError(), 'error while loading not existing guid "' . $hDeleteProductResult->{'guid'} . '", epages 6 object api');
        eval{
            LoadObject($hDeleteProductResult->{'epages6_objectid'});
        };
        ok(ExistsError(), 'error while loading not existing id "' . $hDeleteProductResult->{'epages6_objectid'} . '", epages 6 object api');
    }
    
    # check for empty shop
    $aAllProducts = $Synchronizer->getProductsContentByShopAlias($TestShopAlias);
    
    ok(scalar(@$aAllProducts) == 0, 'shop has no products');
}

sub test_put_products{
    #get variables
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductSynchronization->new();
    
    # check for empty shop
    my $aAllProducts = $Synchronizer->getProductsContentByShopAlias($TestShopAlias);
    
    ok(scalar(@$aAllProducts) == 0, 'shop has no products');
    
    #get test data
    my $aTestPutProductsOriginal = [{
        'alias' => 'MSIntegrationPutTestProduct-01',
        'product_type' => 'Jacket',
        'position' => 17,
        'isvisible' => 'true',
        'visibleinnavigation' => 'true',
        'imagesmall' => 'YD05F_s.JPG',
        'imagemedium' => 'YD05F_m.JPG',
        'taxclass' => 'normal',
        'isnew' => 'true',
        'isdailyprice' => 'false',
        'orderunit' => 'piece',
        'pricequantity' => 1.0,
        'minorder' => 1.0,
        'intervalorder' => 1.0,
        'weightunit' => 'kilogram',
        'weight' => 1.17,
        'refunit' => 'piece',
        'refamount' => 1,
        'refcontentamount' => 1.0,
        'stocklevel' => 3.0,
        'stocklevelalert' => 1.0,
        #'manufacturer' => 'NGK', needs manufacturer synchronization first
        #'manufacturersku' => ???,  needs manufacturer synchronization first
        'deliveryperiod' => 'test',
        'length' => 2.0,
        'height' => 3.0,
        'width' => 4.0,
        'isavailable' => 'true',
        #'availabilitydate' => ???,
        'imagemediumsmall' => 'YD05F_ms.JPG',
        'imagemediumlarge' => 'YD05F_ml.JPG',
        'imagelarge' => 'YD05F.JPG',
        'imagehotdeal' => 'YD05F_h.JPG',
        #'superproductid' => '',
        'defaultsubproduct' => '',
        'upcean' => "0087295184547",
        #'newnessdate' => ???,
        'isappointment' => 'false',
        'canpayappointment' => 'false',
        'durationamount' => 17,
        'durationunit' => 'minute',
        'ratingaverage' => 3.5,
        'prepaymenttype' => 'test_01',
        'prepaymentvalue' => 17.0,
        #'ecoparticipationcategoryid' => ???,
        'isdownloadproduct' => 'false',
        'iscustomizable' => 'false',
        #'expirydate' => ???
        'ismaster' => 'false',
        'hassubownprices' => 'true',
    },
    {
        'alias' => 'MSIntegrationPutTestProduct-02',
        'product_type' => 'Jacket',
        'position' => 0,
        'isvisible' => 'false',
        'visibleinnavigation' => 'false',
        'imagesmall' => '',
        'imagemedium' => '',
        'taxclass' => 'normal',
        'isnew' => 'false',
        'isdailyprice' => 'true',
        'orderunit' => 'piece',
        'pricequantity' => 1.0,
        'minorder' => 1.0,
        'intervalorder' => 1.0,
        'weightunit' => 'kilogram',
        'weight' => 999999.17,
        'refunit' => 'piece',
        'refamount' => 1,
        'refcontentamount' => 1.0,
        'stocklevel' => 999999.0,
        'stocklevelalert' => 17.0,
        #'manufacturer' => 'NGK', needs manufacturer synchronization first
        #'manufacturersku' => ???,  needs manufacturer synchronization first
        'deliveryperiod' => 'test2',
        'length' => 90.0,
        'height' => 110.0,
        'width' => 20000.0,
        'isavailable' => 'true',
        #'availabilitydate' => ???,
        'imagemediumsmall' => '',
        'imagemediumlarge' => '',
        'imagelarge' => '',
        'imagehotdeal' => '',
        #'superproduct' => '',
        'defaultsubproduct' => '',
        'upcean' => "",
        #'newnessdate' => ???,
        'isappointment' => 'true',
        'canpayappointment' => 'true',
        'durationamount' => 0,
        'durationunit' => 'minute',
        'ratingaverage' => 0.5,
        'prepaymenttype' => '',
        'prepaymentvalue' => 0.0,
        #'ecoparticipationcategoryid' => ???,
        'isdownloadproduct' => 'true',
        'iscustomizable' => 'true',
        #'expirydate' => ???
        'ismaster' => 'true',
        'hassubownprices' => 'false',
    }
    ];
    my $aTestPutProductsUpdate = [{
        'position' => 29,
        'isvisible' => 'false',
        'visibleinnavigation' => 'false',
        'imagesmall' => 'YD05F_UPDATE_s.JPG',
        'imagemedium' => 'YD05F_UPDATE_m.JPG',
        'taxclass' => 'reduced',
        'isnew' => 'false',
        'isdailyprice' => 'true',
        'orderunit' => 'gram',
        'pricequantity' => 120.0,
        'minorder' => 120.0,
        'intervalorder' => 2.0,
        'weightunit' => 'milligram',
        'weight' => 9.99,
        'refunit' => 'milligram',
        'refamount' => 2,
        'refcontentamount' => 9.99,
        'stocklevel' => 999999.99,
        'stocklevelalert' => 18.99,
        #'manufacturer' => 'NGK', needs manufacturer synchronization first
        #'manufacturersku' => ???,  needs manufacturer synchronization first
        'deliveryperiod' => 'test_UPDATE',
        'length' => 29.0,
        'height' => 39.0,
        'width' => 49.0,
        'isavailable' => 'false',
        #'availabilitydate' => ???,
        'imagemediumsmall' => 'YD05F_UPDATE_ms.JPG',
        'imagemediumlarge' => 'YD05F_UPDATE_ml.JPG',
        'imagelarge' => 'YD05F_UPDATE.JPG',
        'imagehotdeal' => 'YD05F_UPDATE_h.JPG',
        #'superproductid' => '',
        'defaultsubproduct' => 'test',
        'upcean' => "00999999984547",
        #'newnessdate' => ???,
        'isappointment' => 'true',
        'canpayappointment' => 'true',
        'durationamount' => 39,
        'durationunit' => 'hour',
        'ratingaverage' => 4.5,
        'prepaymenttype' => '',
        'prepaymentvalue' => 97.0,
        #'ecoparticipationcategoryid' => ???,
        'isdownloadproduct' => 'true',
        'iscustomizable' => 'true',
        #'expirydate' => ???
        'ismaster' => 'true',
        'hassubownprices' => 'false',
    },
    {
        'position' => 79,
        'isvisible' => 'true',
        'visibleinnavigation' => 'true',
        'imagesmall' => 'UPDATE_s.png',
        'imagemedium' => 'UPDATE_m.png',
        'taxclass' => 'reduced',
        'isnew' => 'true',
        'isdailyprice' => 'false',
        'orderunit' => 'squaremetre',
        'pricequantity' => 17.0,
        'minorder' => 17.0,
        'intervalorder' => 17.0,
        'weightunit' => 'milligram',
        'weight' => 13.17,
        'refunit' => 'gallon',
        'refamount' => 37,
        'refcontentamount' => 37.0,
        'stocklevel' => 99.0,
        'stocklevelalert' => 10.0,
        #'manufacturer' => 'NGK', needs manufacturer synchronization first
        #'manufacturersku' => ???,  needs manufacturer synchronization first
        'deliveryperiod' => 'test2_UPDATE',
        'length' => 190.0,
        'height' => 1110.0,
        'width' => 120000.0,
        'isavailable' => 'false',
        #'availabilitydate' => ???,
        'imagemediumsmall' => 'UPDATE_sm.png',
        'imagemediumlarge' => 'UPDATE_ml.png',
        'imagelarge' => 'UPDATE_l.png',
        'imagehotdeal' => 'UPDATE_h.png',
        #'superproduct' => '',
        'defaultsubproduct' => 'test',
        'upcean' => "0009999000017779897",
        #'newnessdate' => ???,
        'isappointment' => 'false',
        'canpayappointment' => 'false',
        'durationamount' => 17,
        'durationunit' => 'second',
        'ratingaverage' => 5.0,
        'prepaymenttype' => '',
        'prepaymentvalue' => 3.17,
        #'ecoparticipationcategoryid' => ???,
        'isdownloadproduct' => 'false',
        'iscustomizable' => 'false',
        #'expirydate' => ???
        'ismaster' => 'false',
        'hassubownprices' => 'true',
    }
    ];
    
    #####TEST PRODUCTS#####
    foreach my $hUpdateProduct (@$aTestPutProductsOriginal){
    
        #check if test products not exists
        ok(not (ExistsObjectByPath($TestShopPath . '/Products/' . $hUpdateProduct->{'alias'})), 'test product "' . $hUpdateProduct->{'alias'} . '" not exists');
    }
    
    #add products
    my $aProductsResult = $Synchronizer->addProductsToShop($TestShopAlias, $aTestPutProductsOriginal);
    my $hPutTestProductReturnArrayPosition;
    my $aReturnGUIDs;
    my $Counter = -1;
    
    #remember positions and guids
    foreach my $hResultProduct (@$aProductsResult){
        #increase counter
        $Counter += 1;
        
        #get info
        my $ResultAlias = $hResultProduct->{'alias'};
        my $ResultGUID = $hResultProduct->{'guid'};
        my $SourcePosition = $hTestProductSourceArrayPosition->{$ResultAlias};
        
        #remember position
        $hPutTestProductReturnArrayPosition->{$ResultAlias} = $Counter;
        
        #add guid to test update hash
        $aTestPutProductsUpdate->[$SourcePosition]->{'guid'} = $ResultGUID;
        
        #remember guids
        push(@$aReturnGUIDs, $ResultGUID);
        
    }
    
    #get results from synch api
    my $aPerformanceGetResult = $Synchronizer->getProductsContentByGUIDs($aReturnGUIDs);
    
    #check that original state of products exists right now
    foreach my $hPerformanceGetResultForGUID ( @$aPerformanceGetResult){
    
        #get source positon
        my $SourcePosition = $hTestProductSourceArrayPosition->{$hPerformanceGetResultForGUID->{'alias'}};
        
        #check content
        is($hPerformanceGetResultForGUID->{'alias'}, $aTestPutProductsOriginal->[$SourcePosition]->{'alias'}, 'alias from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'position'}, $aTestPutProductsOriginal->[$SourcePosition]->{'position'}, 'position from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'isvisible'}, $aTestPutProductsOriginal->[$SourcePosition]->{'isvisible'}, 'is visible from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'visibleinnavigation'}, $aTestPutProductsOriginal->[$SourcePosition]->{'visibleinnavigation'}, 'visible in navigation from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'imagesmall'}, $aTestPutProductsOriginal->[$SourcePosition]->{'imagesmall'}, 'imagesmall from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'imagemedium'}, $aTestPutProductsOriginal->[$SourcePosition]->{'imagemedium'}, 'imagemedium from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'taxclass'}, $aTestPutProductsOriginal->[$SourcePosition]->{'taxclass'}, 'taxclass from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'isnew'}, $aTestPutProductsOriginal->[$SourcePosition]->{'isnew'}, 'isnew from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'isdailyprice'}, $aTestPutProductsOriginal->[$SourcePosition]->{'isdailyprice'}, 'daily price from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'orderunit'}, $aTestPutProductsOriginal->[$SourcePosition]->{'orderunit'}, 'order unit from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'pricequantity'}, $aTestPutProductsOriginal->[$SourcePosition]->{'pricequantity'}, 'price quantity from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'minorder'}, $aTestPutProductsOriginal->[$SourcePosition]->{'minorder'}, 'minimal order from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'intervalorder'}, $aTestPutProductsOriginal->[$SourcePosition]->{'intervalorder'}, 'interval order from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'weightunit'}, $aTestPutProductsOriginal->[$SourcePosition]->{'weightunit'}, 'weight unit from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'weight'}, $aTestPutProductsOriginal->[$SourcePosition]->{'weight'}, 'weight from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'refunit'}, $aTestPutProductsOriginal->[$SourcePosition]->{'refunit'}, 'referential unit from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'refamount'}, $aTestPutProductsOriginal->[$SourcePosition]->{'refamount'},'referential amount from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'refcontentamount'}, $aTestPutProductsOriginal->[$SourcePosition]->{'refcontentamount'}, 'referential content amount from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'stocklevel'}, $aTestPutProductsOriginal->[$SourcePosition]->{'stocklevel'}, 'stocklevel from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'stocklevelalert'}, $aTestPutProductsOriginal->[$SourcePosition]->{'stocklevelalert'}, 'stocklevel alert from db is correct, epages 6 connector synch api');
        #is($hPerformanceGetResultForGUID->{'manufacturer'}, $aTestPutProductsOriginal->[$SourcePosition]->{'manufacturer'}, needs manufacturer synchronization first
        #is($hPerformanceGetResultForGUID->{'manufacturersku'}, $aTestPutProductsOriginal->[$SourcePosition]->{'manufacturersku'}???,  needs manufacturer synchronization first
        #is($hPerformanceGetResultForGUID->{'deliveryperiod'}, $aTestPutProductsOriginal->[$SourcePosition]->{'deliveryperiod'}???,
        is($hPerformanceGetResultForGUID->{'length'}, $aTestPutProductsOriginal->[$SourcePosition]->{'length'}, 'length from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'height'}, $aTestPutProductsOriginal->[$SourcePosition]->{'height'}, 'height from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'width'}, $aTestPutProductsOriginal->[$SourcePosition]->{'width'}, 'width from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'isavailable'}, $aTestPutProductsOriginal->[$SourcePosition]->{'isavailable'}, 'is available from db is correct, epages 6 connector synch api');
        #is($hPerformanceGetResultForGUID->{'availabilitydate'}, $aTestPutProductsOriginal->[$SourcePosition]->{'availabilitydate'}???,
        is($hPerformanceGetResultForGUID->{'imagemediumsmall'}, $aTestPutProductsOriginal->[$SourcePosition]->{'imagemediumsmall'}, 'imagemediumsmall from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'imagemediumlarge'}, $aTestPutProductsOriginal->[$SourcePosition]->{'imagemediumlarge'}, 'imagemediumlarge from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'imagelarge'}, $aTestPutProductsOriginal->[$SourcePosition]->{'imagelarge'}, 'imagelarge from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'imagehotdeal'}, $aTestPutProductsOriginal->[$SourcePosition]->{'imagehotdeal'}, 'imagehotdeal from db is correct, epages 6 connector synch api');
        #is($hPerformanceGetResultForGUID->{'superproductid'}, $aTestPutProductsOriginal->[$SourcePosition]->{'superproductid'}???,
        #is($hPerformanceGetResultForGUID->{'defaultsubproductid'}, $aTestPutProductsOriginal->[$SourcePosition]->{'defaultsubproductid'}???,
        ok($hPerformanceGetResultForGUID->{'upcean'} eq $aTestPutProductsOriginal->[$SourcePosition]->{'upcean'}, 'upcean order from db is correct, epages 6 connector synch api');
        #is($hPerformanceGetResultForGUID->{'newnessdate'}, $aTestPutProductsOriginal->[$SourcePosition]->{'newnessdate'} ???,
        is($hPerformanceGetResultForGUID->{'isappointment'}, $aTestPutProductsOriginal->[$SourcePosition]->{'isappointment'}, 'isappointment from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'canpayappointment'}, $aTestPutProductsOriginal->[$SourcePosition]->{'canpayappointment'}, 'can pay appointment from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'durationamount'}, $aTestPutProductsOriginal->[$SourcePosition]->{'durationamount'}, 'duration amount from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'durationunit'}, $aTestPutProductsOriginal->[$SourcePosition]->{'durationunit'}, 'duration unit from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'ratingaverage'}, $aTestPutProductsOriginal->[$SourcePosition]->{'ratingaverage'}, 'rating average from db is correct, epages 6 connector synch api');
        ok($hPerformanceGetResultForGUID->{'prepaymenttype'} eq $aTestPutProductsOriginal->[$SourcePosition]->{'prepaymenttype'}, 'pre payment type from db is correct, epages 6 connector synch api');
        ok($hPerformanceGetResultForGUID->{'prepaymentvalue'} eq $aTestPutProductsOriginal->[$SourcePosition]->{'prepaymentvalue'}, 'pre payment value from db is correct, epages 6 connector synch api');
        #is($hPerformanceGetResultForGUID->{'ecoparticipationcategoryid'}, $aTestPutProductsOriginal->[$SourcePosition]->{'ecoparticipationcategoryid'}???,
        is($hPerformanceGetResultForGUID->{'isdownloadproduct'}, $aTestPutProductsOriginal->[$SourcePosition]->{'isdownloadproduct'}, 'isdownloadproduct from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'iscustomizable'}, $aTestPutProductsOriginal->[$SourcePosition]->{'iscustomizable'}, 'iscustomizable from db is correct, epages 6 connector synch api');
        #is($hPerformanceGetResultForGUID->{'expirydate'}, $aTestPutProductsOriginal->[$SourcePosition]->{'expirydate'} ???
        is($hPerformanceGetResultForGUID->{'ismaster'}, $aTestPutProductsOriginal->[$SourcePosition]->{'ismaster'}, 'ismaster from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'hassubownprices'}, $aTestPutProductsOriginal->[$SourcePosition]->{'hassubownprices'}, 'hassubownprices from db is correct, epages 6 connector synch api');
    }
    
    #update products with synch api
    my $aPutResult = $Synchronizer->updateProductsContentToShop($TestShopAlias,$aTestPutProductsUpdate);
    
    foreach my $hPutResult (@$aPutResult){
        ok((exists $hPutResult->{'guid'}) and (exists $hPutResult->{'status'}), 'update result exists');
        is($hPutResult->{'status'}, 'success', 'update result successful');
    }
    
    #get results from synch api
    my $aPerformanceGetResult2nd = $Synchronizer->getProductsContentByGUIDs($aReturnGUIDs);
    
    #check that original state of products exists right now
    foreach my $hPerformanceGetResultForGUID ( @$aPerformanceGetResult2nd){
    
        #get source positon
        my $SourcePosition = $hTestProductSourceArrayPosition->{$hPerformanceGetResultForGUID->{'alias'}};
        
        #check content
        is($hPerformanceGetResultForGUID->{'alias'}, $aTestPutProductsOriginal->[$SourcePosition]->{'alias'}, 'alias from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'position'}, $aTestPutProductsUpdate->[$SourcePosition]->{'position'}, 'position from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'isvisible'}, $aTestPutProductsUpdate->[$SourcePosition]->{'isvisible'}, 'is visible from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'visibleinnavigation'}, $aTestPutProductsUpdate->[$SourcePosition]->{'visibleinnavigation'}, 'visible in navigation from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'imagesmall'}, $aTestPutProductsUpdate->[$SourcePosition]->{'imagesmall'}, 'imagesmall from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'imagemedium'}, $aTestPutProductsUpdate->[$SourcePosition]->{'imagemedium'}, 'imagemedium from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'taxclass'}, $aTestPutProductsUpdate->[$SourcePosition]->{'taxclass'}, 'taxclass from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'isnew'}, $aTestPutProductsUpdate->[$SourcePosition]->{'isnew'}, 'isnew from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'isdailyprice'}, $aTestPutProductsUpdate->[$SourcePosition]->{'isdailyprice'}, 'daily price from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'orderunit'}, $aTestPutProductsUpdate->[$SourcePosition]->{'orderunit'}, 'order unit from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'pricequantity'}, $aTestPutProductsUpdate->[$SourcePosition]->{'pricequantity'}, 'price quantity from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'minorder'}, $aTestPutProductsUpdate->[$SourcePosition]->{'minorder'}, 'minimal order from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'intervalorder'}, $aTestPutProductsUpdate->[$SourcePosition]->{'intervalorder'}, 'interval order from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'weightunit'}, $aTestPutProductsUpdate->[$SourcePosition]->{'weightunit'}, 'weight unit from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'weight'}, $aTestPutProductsUpdate->[$SourcePosition]->{'weight'}, 'weight from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'refunit'}, $aTestPutProductsUpdate->[$SourcePosition]->{'refunit'}, 'referential unit from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'refamount'}, $aTestPutProductsUpdate->[$SourcePosition]->{'refamount'},'referential amount from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'refcontentamount'}, $aTestPutProductsUpdate->[$SourcePosition]->{'refcontentamount'}, 'referential content amount from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'stocklevel'}, $aTestPutProductsUpdate->[$SourcePosition]->{'stocklevel'}, 'stocklevel from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'stocklevelalert'}, $aTestPutProductsUpdate->[$SourcePosition]->{'stocklevelalert'}, 'stocklevel alert from db is correct, epages 6 connector synch api');
        #is($hPerformanceGetResultForGUID->{'manufacturer'}, $aTestPutProductsUpdate->[$SourcePosition]->{'manufacturer'}, needs manufacturer synchronization first
        #is($hPerformanceGetResultForGUID->{'manufacturersku'}, $aTestPutProductsUpdate->[$SourcePosition]->{'manufacturersku'}???,  needs manufacturer synchronization first
        #is($hPerformanceGetResultForGUID->{'deliveryperiod'}, $aTestPutProductsUpdate->[$SourcePosition]->{'deliveryperiod'}???,
        is($hPerformanceGetResultForGUID->{'length'}, $aTestPutProductsUpdate->[$SourcePosition]->{'length'}, 'length from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'height'}, $aTestPutProductsUpdate->[$SourcePosition]->{'height'}, 'height from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'width'}, $aTestPutProductsUpdate->[$SourcePosition]->{'width'}, 'width from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'isavailable'}, $aTestPutProductsUpdate->[$SourcePosition]->{'isavailable'}, 'is available from db is correct, epages 6 connector synch api');
        #is($hPerformanceGetResultForGUID->{'availabilitydate'}, $aTestPutProductsUpdate->[$SourcePosition]->{'availabilitydate'}???,
        is($hPerformanceGetResultForGUID->{'imagemediumsmall'}, $aTestPutProductsUpdate->[$SourcePosition]->{'imagemediumsmall'}, 'imagemediumsmall from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'imagemediumlarge'}, $aTestPutProductsUpdate->[$SourcePosition]->{'imagemediumlarge'}, 'imagemediumlarge from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'imagelarge'}, $aTestPutProductsUpdate->[$SourcePosition]->{'imagelarge'}, 'imagelarge from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'imagehotdeal'}, $aTestPutProductsUpdate->[$SourcePosition]->{'imagehotdeal'}, 'imagehotdeal from db is correct, epages 6 connector synch api');
        #is($hPerformanceGetResultForGUID->{'superproductid'}, $aTestPutProductsUpdate->[$SourcePosition]->{'superproductid'}???,
        #is($hPerformanceGetResultForGUID->{'defaultsubproductid'}, $aTestPutProductsUpdate->[$SourcePosition]->{'defaultsubproductid'}???,
        ok($hPerformanceGetResultForGUID->{'upcean'} eq $aTestPutProductsUpdate->[$SourcePosition]->{'upcean'}, 'upcean order from db is correct, epages 6 connector synch api');
        #is($hPerformanceGetResultForGUID->{'newnessdate'}, $aTestPutProductsUpdate->[$SourcePosition]->{'newnessdate'} ???,
        is($hPerformanceGetResultForGUID->{'isappointment'}, $aTestPutProductsUpdate->[$SourcePosition]->{'isappointment'}, 'isappointment from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'canpayappointment'}, $aTestPutProductsUpdate->[$SourcePosition]->{'canpayappointment'}, 'can pay appointment from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'durationamount'}, $aTestPutProductsUpdate->[$SourcePosition]->{'durationamount'}, 'duration amount from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'durationunit'}, $aTestPutProductsUpdate->[$SourcePosition]->{'durationunit'}, 'duration unit from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'ratingaverage'}, $aTestPutProductsUpdate->[$SourcePosition]->{'ratingaverage'}, 'rating average from db is correct, epages 6 connector synch api');
        ok($hPerformanceGetResultForGUID->{'prepaymenttype'} eq $aTestPutProductsUpdate->[$SourcePosition]->{'prepaymenttype'}, 'pre payment type from db is correct, epages 6 connector synch api');
        ok($hPerformanceGetResultForGUID->{'prepaymentvalue'} eq $aTestPutProductsUpdate->[$SourcePosition]->{'prepaymentvalue'}, 'pre payment value from db is correct, epages 6 connector synch api');
        #is($hPerformanceGetResultForGUID->{'ecoparticipationcategoryid'}, $aTestPutProductsUpdate->[$SourcePosition]->{'ecoparticipationcategoryid'}???,
        is($hPerformanceGetResultForGUID->{'isdownloadproduct'}, $aTestPutProductsUpdate->[$SourcePosition]->{'isdownloadproduct'}, 'isdownloadproduct from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'iscustomizable'}, $aTestPutProductsUpdate->[$SourcePosition]->{'iscustomizable'}, 'iscustomizable from db is correct, epages 6 connector synch api');
        #is($hPerformanceGetResultForGUID->{'expirydate'}, $aTestPutProductsUpdate->[$SourcePosition]->{'expirydate'} ???
        is($hPerformanceGetResultForGUID->{'ismaster'}, $aTestPutProductsUpdate->[$SourcePosition]->{'ismaster'}, 'ismaster from db is correct, epages 6 connector synch api');
        is($hPerformanceGetResultForGUID->{'hassubownprices'}, $aTestPutProductsUpdate->[$SourcePosition]->{'hassubownprices'}, 'hassubownprices from db is correct, epages 6 connector synch api');
    }
    
    # delete products
    delete_all_products();
    
    # check for empty shop
    $aAllProducts = $Synchronizer->getProductsContentByShopAlias($TestShopAlias);
    
    ok(scalar(@$aAllProducts) == 0, 'shop has no products');
}

sub test_get_products_pagination{
    #get variables
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductSynchronization->new();
    
    # check for empty shop
    my $aAllProducts = $Synchronizer->getProductsContentByShopAlias($TestShopAlias);
    
    ok(scalar(@$aAllProducts) == 0, 'shop has no products');
    
    #get test data
    my $aTestGetProductsPagination = [{
            'alias' => 'MSIntegrationGetPaginationTestProduct-01',
            'product_type' => 'Jacket',
            'position' => 1,
            'isvisible' => 'true',
            'visibleinnavigation' => 'true',
            'imagesmall' => 'YD05F_s.JPG',
            'imagemedium' => 'YD05F_m.JPG',
            'taxclass' => 'normal',
            'isnew' => 'true',
            'isdailyprice' => 'false',
            'orderunit' => 'piece',
            'pricequantity' => 1.0,
            'minorder' => 1.0,
            'intervalorder' => 1.0,
            'weightunit' => 'kilogram',
            'weight' => 1.17,
            'refunit' => 'piece',
            'refamount' => 1,
            'refcontentamount' => 1.0,
            'stocklevel' => 3.0,
            'stocklevelalert' => 1.0,
            'deliveryperiod' => 'test',
            'length' => 2.0,
            'height' => 3.0,
            'width' => 4.0,
            'isavailable' => 'true',
            'imagemediumsmall' => 'YD05F_ms.JPG',
            'imagemediumlarge' => 'YD05F_ml.JPG',
            'imagelarge' => 'YD05F.JPG',
            'imagehotdeal' => 'YD05F_h.JPG',
            'defaultsubproduct' => '',
            'upcean' => "0087295184547",
            'isappointment' => 'false',
            'canpayappointment' => 'false',
            'durationamount' => 17,
            'durationunit' => 'minute',
            'ratingaverage' => 3.5,
            'prepaymenttype' => 'test_01',
            'prepaymentvalue' => 17.0,
            'isdownloadproduct' => 'false',
            'iscustomizable' => 'false',
            'ismaster' => 'false',
            'hassubownprices' => 'true',
        },
        {
            'alias' => 'MSIntegrationGetPaginationTestProduct-02',
            'product_type' => 'Jacket',
            'position' => 2,
            'isvisible' => 'false',
            'visibleinnavigation' => 'false',
            'imagesmall' => '',
            'imagemedium' => '',
            'taxclass' => 'normal',
            'isnew' => 'false',
            'isdailyprice' => 'true',
            'orderunit' => 'piece',
            'pricequantity' => 1.0,
            'minorder' => 1.0,
            'intervalorder' => 1.0,
            'weightunit' => 'kilogram',
            'weight' => 999999.17,
            'refunit' => 'piece',
            'refamount' => 1,
            'refcontentamount' => 1.0,
            'stocklevel' => 999999.0,
            'stocklevelalert' => 17.0,
            'deliveryperiod' => 'test2',
            'length' => 90.0,
            'height' => 110.0,
            'width' => 20000.0,
            'isavailable' => 'true',
            'imagemediumsmall' => '',
            'imagemediumlarge' => '',
            'imagelarge' => '',
            'imagehotdeal' => '',
            'defaultsubproduct' => '',
            'upcean' => "",
            'isappointment' => 'true',
            'canpayappointment' => 'true',
            'durationamount' => 0,
            'durationunit' => 'minute',
            'ratingaverage' => 0.5,
            'prepaymenttype' => '',
            'prepaymentvalue' => 0.0,
            'isdownloadproduct' => 'true',
            'iscustomizable' => 'true',
            'ismaster' => 'true',
            'hassubownprices' => 'false',
        },
        {
            'alias' => 'MSIntegrationGetPaginationTestProduct-03',
            'product_type' => 'Jacket',
            'position' => 3,
            'isvisible' => 'true',
            'visibleinnavigation' => 'true',
            'imagesmall' => 'YD05F_s.JPG',
            'imagemedium' => 'YD05F_m.JPG',
            'taxclass' => 'normal',
            'isnew' => 'true',
            'isdailyprice' => 'false',
            'orderunit' => 'piece',
            'pricequantity' => 1.0,
            'minorder' => 1.0,
            'intervalorder' => 1.0,
            'weightunit' => 'kilogram',
            'weight' => 1.17,
            'refunit' => 'piece',
            'refamount' => 1,
            'refcontentamount' => 1.0,
            'stocklevel' => 3.0,
            'stocklevelalert' => 1.0,
            'deliveryperiod' => 'test',
            'length' => 2.0,
            'height' => 3.0,
            'width' => 4.0,
            'isavailable' => 'true',
            'imagemediumsmall' => 'YD05F_ms.JPG',
            'imagemediumlarge' => 'YD05F_ml.JPG',
            'imagelarge' => 'YD05F.JPG',
            'imagehotdeal' => 'YD05F_h.JPG',
            'defaultsubproduct' => '',
            'upcean' => "0087295184547",
            'isappointment' => 'false',
            'canpayappointment' => 'false',
            'durationamount' => 17,
            'durationunit' => 'minute',
            'ratingaverage' => 3.5,
            'prepaymenttype' => 'test_01',
            'prepaymentvalue' => 17.0,
            'isdownloadproduct' => 'false',
            'iscustomizable' => 'false',
            'ismaster' => 'false',
            'hassubownprices' => 'true',
        },
        {
            'alias' => 'MSIntegrationGetPaginationTestProduct-04',
            'product_type' => 'Jacket',
            'position' => 4,
            'isvisible' => 'false',
            'visibleinnavigation' => 'false',
            'imagesmall' => '',
            'imagemedium' => '',
            'taxclass' => 'normal',
            'isnew' => 'false',
            'isdailyprice' => 'true',
            'orderunit' => 'piece',
            'pricequantity' => 1.0,
            'minorder' => 1.0,
            'intervalorder' => 1.0,
            'weightunit' => 'kilogram',
            'weight' => 999999.17,
            'refunit' => 'piece',
            'refamount' => 1,
            'refcontentamount' => 1.0,
            'stocklevel' => 999999.0,
            'stocklevelalert' => 17.0,
            'deliveryperiod' => 'test2',
            'length' => 90.0,
            'height' => 110.0,
            'width' => 20000.0,
            'isavailable' => 'true',
            'imagemediumsmall' => '',
            'imagemediumlarge' => '',
            'imagelarge' => '',
            'imagehotdeal' => '',
            'defaultsubproduct' => '',
            'upcean' => "",
            'isappointment' => 'true',
            'canpayappointment' => 'true',
            'durationamount' => 0,
            'durationunit' => 'minute',
            'ratingaverage' => 0.5,
            'prepaymenttype' => '',
            'prepaymentvalue' => 0.0,
            'isdownloadproduct' => 'true',
            'iscustomizable' => 'true',
            'ismaster' => 'true',
            'hassubownprices' => 'false',
        },
        {
            'alias' => 'MSIntegrationGetPaginationTestProduct-05',
            'product_type' => 'Jacket',
            'position' => 5,
            'isvisible' => 'true',
            'visibleinnavigation' => 'true',
            'imagesmall' => 'YD05F_s.JPG',
            'imagemedium' => 'YD05F_m.JPG',
            'taxclass' => 'normal',
            'isnew' => 'true',
            'isdailyprice' => 'false',
            'orderunit' => 'piece',
            'pricequantity' => 1.0,
            'minorder' => 1.0,
            'intervalorder' => 1.0,
            'weightunit' => 'kilogram',
            'weight' => 1.17,
            'refunit' => 'piece',
            'refamount' => 1,
            'refcontentamount' => 1.0,
            'stocklevel' => 3.0,
            'stocklevelalert' => 1.0,
            'deliveryperiod' => 'test',
            'length' => 2.0,
            'height' => 3.0,
            'width' => 4.0,
            'isavailable' => 'true',
            'imagemediumsmall' => 'YD05F_ms.JPG',
            'imagemediumlarge' => 'YD05F_ml.JPG',
            'imagelarge' => 'YD05F.JPG',
            'imagehotdeal' => 'YD05F_h.JPG',
            'defaultsubproduct' => '',
            'upcean' => "0087295184547",
            'isappointment' => 'false',
            'canpayappointment' => 'false',
            'durationamount' => 17,
            'durationunit' => 'minute',
            'ratingaverage' => 3.5,
            'prepaymenttype' => 'test_01',
            'prepaymentvalue' => 17.0,
            'isdownloadproduct' => 'false',
            'iscustomizable' => 'false',
            'ismaster' => 'false',
            'hassubownprices' => 'true',
        },
        {
            'alias' => 'MSIntegrationGetPaginationTestProduct-06',
            'product_type' => 'Jacket',
            'position' => 6,
            'isvisible' => 'false',
            'visibleinnavigation' => 'false',
            'imagesmall' => '',
            'imagemedium' => '',
            'taxclass' => 'normal',
            'isnew' => 'false',
            'isdailyprice' => 'true',
            'orderunit' => 'piece',
            'pricequantity' => 1.0,
            'minorder' => 1.0,
            'intervalorder' => 1.0,
            'weightunit' => 'kilogram',
            'weight' => 999999.17,
            'refunit' => 'piece',
            'refamount' => 1,
            'refcontentamount' => 1.0,
            'stocklevel' => 999999.0,
            'stocklevelalert' => 17.0,
            'deliveryperiod' => 'test2',
            'length' => 90.0,
            'height' => 110.0,
            'width' => 20000.0,
            'isavailable' => 'true',
            'imagemediumsmall' => '',
            'imagemediumlarge' => '',
            'imagelarge' => '',
            'imagehotdeal' => '',
            'defaultsubproduct' => '',
            'upcean' => "",
            'isappointment' => 'true',
            'canpayappointment' => 'true',
            'durationamount' => 0,
            'durationunit' => 'minute',
            'ratingaverage' => 0.5,
            'prepaymenttype' => '',
            'prepaymentvalue' => 0.0,
            'isdownloadproduct' => 'true',
            'iscustomizable' => 'true',
            'ismaster' => 'true',
            'hassubownprices' => 'false',
        },
        {
            'alias' => 'MSIntegrationGetPaginationTestProduct-07',
            'product_type' => 'Jacket',
            'position' => 7,
            'isvisible' => 'true',
            'visibleinnavigation' => 'true',
            'imagesmall' => 'YD05F_s.JPG',
            'imagemedium' => 'YD05F_m.JPG',
            'taxclass' => 'normal',
            'isnew' => 'true',
            'isdailyprice' => 'false',
            'orderunit' => 'piece',
            'pricequantity' => 1.0,
            'minorder' => 1.0,
            'intervalorder' => 1.0,
            'weightunit' => 'kilogram',
            'weight' => 1.17,
            'refunit' => 'piece',
            'refamount' => 1,
            'refcontentamount' => 1.0,
            'stocklevel' => 3.0,
            'stocklevelalert' => 1.0,
            'deliveryperiod' => 'test',
            'length' => 2.0,
            'height' => 3.0,
            'width' => 4.0,
            'isavailable' => 'true',
            'imagemediumsmall' => 'YD05F_ms.JPG',
            'imagemediumlarge' => 'YD05F_ml.JPG',
            'imagelarge' => 'YD05F.JPG',
            'imagehotdeal' => 'YD05F_h.JPG',
            'defaultsubproduct' => '',
            'upcean' => "0087295184547",
            'isappointment' => 'false',
            'canpayappointment' => 'false',
            'durationamount' => 17,
            'durationunit' => 'minute',
            'ratingaverage' => 3.5,
            'prepaymenttype' => 'test_01',
            'prepaymentvalue' => 17.0,
            'isdownloadproduct' => 'false',
            'iscustomizable' => 'false',
            'ismaster' => 'false',
            'hassubownprices' => 'true',
        },
        {
            'alias' => 'MSIntegrationGetPaginationTestProduct-08',
            'product_type' => 'Jacket',
            'position' => 8,
            'isvisible' => 'false',
            'visibleinnavigation' => 'false',
            'imagesmall' => '',
            'imagemedium' => '',
            'taxclass' => 'normal',
            'isnew' => 'false',
            'isdailyprice' => 'true',
            'orderunit' => 'piece',
            'pricequantity' => 1.0,
            'minorder' => 1.0,
            'intervalorder' => 1.0,
            'weightunit' => 'kilogram',
            'weight' => 999999.17,
            'refunit' => 'piece',
            'refamount' => 1,
            'refcontentamount' => 1.0,
            'stocklevel' => 999999.0,
            'stocklevelalert' => 17.0,
            'deliveryperiod' => 'test2',
            'length' => 90.0,
            'height' => 110.0,
            'width' => 20000.0,
            'isavailable' => 'true',
            'imagemediumsmall' => '',
            'imagemediumlarge' => '',
            'imagelarge' => '',
            'imagehotdeal' => '',
            'defaultsubproduct' => '',
            'upcean' => "",
            'isappointment' => 'true',
            'canpayappointment' => 'true',
            'durationamount' => 0,
            'durationunit' => 'minute',
            'ratingaverage' => 0.5,
            'prepaymenttype' => '',
            'prepaymentvalue' => 0.0,
            'isdownloadproduct' => 'true',
            'iscustomizable' => 'true',
            'ismaster' => 'true',
            'hassubownprices' => 'false',
        },
        {
            'alias' => 'MSIntegrationGetPaginationTestProduct-09',
            'product_type' => 'Jacket',
            'position' => 9,
            'isvisible' => 'true',
            'visibleinnavigation' => 'true',
            'imagesmall' => 'YD05F_s.JPG',
            'imagemedium' => 'YD05F_m.JPG',
            'taxclass' => 'normal',
            'isnew' => 'true',
            'isdailyprice' => 'false',
            'orderunit' => 'piece',
            'pricequantity' => 1.0,
            'minorder' => 1.0,
            'intervalorder' => 1.0,
            'weightunit' => 'kilogram',
            'weight' => 1.17,
            'refunit' => 'piece',
            'refamount' => 1,
            'refcontentamount' => 1.0,
            'stocklevel' => 3.0,
            'stocklevelalert' => 1.0,
            'deliveryperiod' => 'test',
            'length' => 2.0,
            'height' => 3.0,
            'width' => 4.0,
            'isavailable' => 'true',
            'imagemediumsmall' => 'YD05F_ms.JPG',
            'imagemediumlarge' => 'YD05F_ml.JPG',
            'imagelarge' => 'YD05F.JPG',
            'imagehotdeal' => 'YD05F_h.JPG',
            'defaultsubproduct' => '',
            'upcean' => "0087295184547",
            'isappointment' => 'false',
            'canpayappointment' => 'false',
            'durationamount' => 17,
            'durationunit' => 'minute',
            'ratingaverage' => 3.5,
            'prepaymenttype' => 'test_01',
            'prepaymentvalue' => 17.0,
            'isdownloadproduct' => 'false',
            'iscustomizable' => 'false',
            'ismaster' => 'false',
            'hassubownprices' => 'true',
        },
        {
            'alias' => 'MSIntegrationGetPaginationTestProduct-10',
            'product_type' => 'Jacket',
            'position' => 10,
            'isvisible' => 'false',
            'visibleinnavigation' => 'false',
            'imagesmall' => '',
            'imagemedium' => '',
            'taxclass' => 'normal',
            'isnew' => 'false',
            'isdailyprice' => 'true',
            'orderunit' => 'piece',
            'pricequantity' => 1.0,
            'minorder' => 1.0,
            'intervalorder' => 1.0,
            'weightunit' => 'kilogram',
            'weight' => 999999.17,
            'refunit' => 'piece',
            'refamount' => 1,
            'refcontentamount' => 1.0,
            'stocklevel' => 999999.0,
            'stocklevelalert' => 17.0,
            'deliveryperiod' => 'test2',
            'length' => 90.0,
            'height' => 110.0,
            'width' => 20000.0,
            'isavailable' => 'true',
            'imagemediumsmall' => '',
            'imagemediumlarge' => '',
            'imagelarge' => '',
            'imagehotdeal' => '',
            'defaultsubproduct' => '',
            'upcean' => "",
            'isappointment' => 'true',
            'canpayappointment' => 'true',
            'durationamount' => 0,
            'durationunit' => 'minute',
            'ratingaverage' => 0.5,
            'prepaymenttype' => '',
            'prepaymentvalue' => 0.0,
            'isdownloadproduct' => 'true',
            'iscustomizable' => 'true',
            'ismaster' => 'true',
            'hassubownprices' => 'false',
        },
        {
            'alias' => 'MSIntegrationGetPaginationTestProduct-11',
            'product_type' => 'Jacket',
            'position' => 11,
            'isvisible' => 'false',
            'visibleinnavigation' => 'false',
            'imagesmall' => '',
            'imagemedium' => '',
            'taxclass' => 'normal',
            'isnew' => 'false',
            'isdailyprice' => 'true',
            'orderunit' => 'piece',
            'pricequantity' => 1.0,
            'minorder' => 1.0,
            'intervalorder' => 1.0,
            'weightunit' => 'kilogram',
            'weight' => 999999.17,
            'refunit' => 'piece',
            'refamount' => 1,
            'refcontentamount' => 1.0,
            'stocklevel' => 999999.0,
            'stocklevelalert' => 17.0,
            'deliveryperiod' => 'test2',
            'length' => 90.0,
            'height' => 110.0,
            'width' => 20000.0,
            'isavailable' => 'true',
            'imagemediumsmall' => '',
            'imagemediumlarge' => '',
            'imagelarge' => '',
            'imagehotdeal' => '',
            'defaultsubproduct' => '',
            'upcean' => "",
            'isappointment' => 'true',
            'canpayappointment' => 'true',
            'durationamount' => 0,
            'durationunit' => 'minute',
            'ratingaverage' => 0.5,
            'prepaymenttype' => '',
            'prepaymentvalue' => 0.0,
            'isdownloadproduct' => 'true',
            'iscustomizable' => 'true',
            'ismaster' => 'true',
            'hassubownprices' => 'false',
        }
    ];
    
    #####NOT EXISTS PRODUCTS#####
    foreach my $hGetPaginationProduct (@$aTestGetProductsPagination){
    
        #check if test products not exists
        ok(not (ExistsObjectByPath($TestShopPath . '/Products/' . $hGetPaginationProduct->{'alias'})), 'test product "' . $hGetPaginationProduct->{'alias'} . '" not exists');
    }
    
    #add products
    $Synchronizer->addProductsToShop($TestShopAlias, $aTestGetProductsPagination);
    
    # check for product count
    $aAllProducts = $Synchronizer->getProductsContentByShopAlias($TestShopAlias);
    
    ok(scalar(@$aAllProducts) == 11, 'shop has no products');
    
    my $aResultPerPage = $Synchronizer->getProductsContentByShopAliasPaged($TestShopAlias, 0, 5);
    
    ok((scalar(grep {$_} @$aResultPerPage) == 5) , 'exact 5 products');
    
    $aResultPerPage = $Synchronizer->getProductsContentByShopAliasPaged($TestShopAlias, 1, 5);
    
    ok((scalar(grep {$_} @$aResultPerPage) == 5) , 'exact 5 products on 2nd page');
    
    $aResultPerPage = $Synchronizer->getProductsContentByShopAliasPaged($TestShopAlias, 2, 5);

    ok((scalar(grep {$_} @$aResultPerPage) == 1) , 'exact 1 products on 3nd page');
    
    $aResultPerPage = $Synchronizer->getProductsContentByShopAliasPaged($TestShopAlias, 3, 5);

    ok((scalar(grep {$_} @$aResultPerPage) == 0) , 'exact 0 products on 4th page');
    
}

sub test_rollback {

}

sub test {
    
    #test attributes parameter types function
    test_parameter_type();
    
    #test attributes mandatory function
    test_mandatory();
    
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            
            #check if shop exists
            ok(ExistsObjectByPath($TestShopPath), 'integration test shop exists');
            
            #test normal product
            test_products();
            
            #test post products focus on product types assignment
            test_post_products_product_type_assignment();
            
            #test non products
            test_non_product_guids();
            
            #test sql injections
            test_sql_injection();
            
            #test null value products
            test_null_value_products();
            
            #test delete products
            test_delete_products();
            
            #test put update
            test_put_products();
            
            #test get products pagination
            test_get_products_pagination();
            
            done_testing();
        }
    );
}

sub cleanup {
    #delete integration test shop if exists
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            LoadObjectByPath($TestShopPath)->delete if ExistsObjectByPath($TestShopPath);
            DeleteHooks($HookXML);
        }
    );

    return 1;
}

sub delete_all_products {
    my $aGUIDs;
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductSynchronization->new();
    my $aAllProducts = $Synchronizer->getProductsContentByShopAlias($TestShopAlias);
    
    # for each result
    foreach my $hProduct (@$aAllProducts){
        push(@$aGUIDs, $hProduct->{'guid'});
    }
    
    # delete products
    $Synchronizer->deleteProductsByGUIDs($aGUIDs);
}

sub init {
    #add integration test store
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            ImportObjects( $TestShopXML, '/Shops' );
            ImportHooks($HookXML);
        }
    );

    # reset counter for hook calls
    ResetCountersProduct();

    return 1;
}

__PACKAGE__->runScript;
