use Test::Simple tests => 4;

use strict;
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw( fcmp );
use DE_EPAGES::Order::API::HirePurchase qw( CalculateRate CalculateEffectiveInterestRate );


# berechne die mon. rate bei folgenden bedingungen

my $Months = 36;
my $Sum = 1559;
my $Zins = 8.2; # ist der Nominalzins


my $amount = CalculateRate($Sum, $Months, $Zins/100);
ok( 0== fcmp(49,$amount,2) , 'monthly rate' );
#print "$amount\n";

# und nun den effektiven Jahreszins aus den errechneten Wert .....
my $effzins = CalculateEffectiveInterestRate ($Sum, $amount, $Months, $Zins);
ok( 0== fcmp(8.515,$effzins,3) , 'effective interest rate' );
#print "$effzins\n";

# Test Case EPG-9209
eval {
    CalculateEffectiveInterestRate( $Sum, $amount, 0, $Zins );
};
ok( ExistsError, 'Expected error with $Month= 0 was thrown.' );

SKIP:
{
    skip( 'because no error was thrown with $Month= 0.', 1 ) if not ExistsError;

    my $error_message= GetError->message;
    ok( $error_message=~ m/^Months is too small \(0 < 1\)/, 'Error message starts with "Months is too small (0 < 1)".' );
}

#----------------------------------------------------------------------------------
#Berechnung des effektiven Jahreszins:

#Quellen:
#http://bundesrecht.juris.de/pangv/BJNR105800985.html
#http://www.bundesrecht.juris.de/pangv/anhang_12.html

#siehe auch EPG-6134 Anzeige des effektiven Jahreszinses bei Ratenzahlung


#Eine einfache Formal gibt es nicht - es muss iterativ berechnet werden.
# Die Berechnung entspricht der Berechnung nach der Interne-Zinsfu-Methode
# http://de.wikipedia.org/wiki/Interner_Zinsfu%C3%9F
# und http://de.wikipedia.org/wiki/Effektiver_Jahreszins


#Ausgangswerte sind die monatliche Rate (mein Beispiel: 49,00 ), der zu finanzierende Betrag (1559,00 ) und die Laufzeit (36 Monate) sowie der (Nominal-)Zins (8,2%).

#Die Formel:
#FB = R / ((1 + i)^(U/12))
#mit:
#FB - zu finanzierender Betrag (1559,00 )
#R - monatlich Rate (49,00 )
#i - effektiver Jahreszins (die gesuchte Gre)
#U - Monate nach der Auszahlung
#
#Man beginnt mit dem Nominalzins (8,2%) als i und erhlt im ersten Schritt natrlich ein zu 1559,00  abweichendes Ergebnis. Da der effektive Jahreszins ber dem Nominalzins liegt, erhht man nun iterativ, bis das Ergebnis auf zwei Stellen gerundet dem FB (1559,00 ) entspricht. Lt. PAngV wird auch der effekitve Jahreszins auf zwei Stellen nach dem Komma gerundet angezeigt. In unserem Beispiel liegt der genaue Wert bei 8,5193%, gerundet also 8,52%.
