#========================================================================================
# package      DE_EPAGES::SageOne::t::unit::ShopService_AuthUrl
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::SageOne::API::Webservice::ShopService
#========================================================================================
package DE_EPAGES::SageOne::t::unit::ShopService_AuthUrl;
use base DE_EPAGES::Test::API::UnitTest::Test;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::SageOne::API::DTO::AuthUrl;
use DE_EPAGES::Core::API::Error qw (Error);
use Test::More;
use Test::Exception;

use strict;

sub test {
    my $self = shift;

    $self->_buildMocks();

    $self->test_DoRequest_Successful();
    $self->test_DoRequest_WebserviceError();
    $self->test_DoRequest_ResponseError();
    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_DoRequest_Successful {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory()->get('SUT_Successful');

    my $Ep6RedirectUri = 'http://epages6.com/url';
    my $EPjRedirectUri = 'http://epagesj.com/url';
    my $AuthUrlDTO = $SUT->getAuthUrl($Ep6RedirectUri, $EPjRedirectUri);

    is($AuthUrlDTO->authUrl(), 'http://connect.url', "$SubTestName - test if request was successful and return correct DTO");
}

sub test_DoRequest_WebserviceError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory()->get('SUT_WebserviceError');

    my $Ep6RedirectUri = 'http://epages6.com/url';
    my $EPjRedirectUri = 'http://epagesj.com/url';

    throws_ok(sub { $SUT->getAuthUrl($Ep6RedirectUri, $EPjRedirectUri); }, qr/WebserviceError/, "$SubTestName - test if getAuthUrl throws WebserviceError");
}

sub test_DoRequest_ResponseError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory()->get('SUT_ResponseError');

    my $Ep6RedirectUri = 'http://epages6.com/url';
    my $EPjRedirectUri = 'http://epagesj.com/url';

    throws_ok(sub { $SUT->getAuthUrl($Ep6RedirectUri, $EPjRedirectUri); }, qr/ResponseError: Missing Field auth_url/, "$SubTestName - test if getAuthUrl throws ResponseError");
}
#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _buildMocks {
    my $self = shift;
    my $Shop = $self->mockFactory()->buildShop({
        'GUID'=>'MyShopAlias',
        'RESTAPIURL' => 'http://epagesj/base/url/',
        })->getMockObject();
    my $System = $self->mockFactory->buildSite({
        'EPagesJBaseURL' => 'http://unstable-main.epages.com:8088/rs/'
    })->getMockObject();
    my $SUT_Successful = $self->_createSUTSuccessful($Shop, $System);
    my $SUT_WebserviceError = $self->_createSUTWebserviceError($Shop, $System);
    my $SUT_ResponseError = $self->_createSUTResponseError($Shop, $System);

    $self->memory()->add('SUT_Successful', $SUT_Successful);
    $self->memory()->add('SUT_WebserviceError', $SUT_WebserviceError);
    $self->memory()->add('SUT_ResponseError', $SUT_ResponseError);
}
#----------------------------------------------------------------------------------------
sub _createSUTSuccessful {
    my $self = shift;
    my ($Shop, $System) = @_;

    my $aParameterList = [$Shop];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::ShopService',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('_createShopUriBuilder', $self->_createShopUriBuilder($Shop, $System), [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]);
    $Mockify->addMockWithReturnValueAndParameterCheck('sendGetRequest',
       {'code' => 200, 'content' => {'auth_url' => 'http://connect.url'}},
       [{'string' => 'http://baseuri.com'}, {'hashref'=> {}}]);
    $Mockify->addMockWithReturnValueAndParameterCheck('decodeResponse',
       {'auth_url' => 'http://connect.url'},
       [{'hashref'=> {'code' => 200, 'content' => {'auth_url' => 'http://connect.url'}}}]
    );

    return $Mockify->getMockObject();
}

sub _createSUTWebserviceError {
    my $self = shift;
    my ($Shop, $System) = @_;

    my $aParameterList = [$Shop];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::ShopService',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('_createShopUriBuilder', $self->_createShopUriBuilder($Shop, $System), [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]);
    $Mockify->addMock('sendGetRequest', sub{ Error('WebserviceError');});

    return $Mockify->getMockObject();
}

sub _createSUTResponseError {
    my $self = shift;
    my ($Shop, $System) = @_;

    my $Response = {'code' => 200, 'content' => {'another' => 'http://connect.url'}};
    my $aParameterList = [$Shop];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::ShopService',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('_createShopUriBuilder', $self->_createShopUriBuilder($Shop, $System), [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]);
    $Mockify->addMockWithReturnValueAndParameterCheck('sendGetRequest', $Response, [{'string' => 'http://baseuri.com'}, {'hashref'=> {}}]);
    $Mockify->addMockWithReturnValueAndParameterCheck('decodeResponse',
       {'another' => 'http://connect.url'}, [{'hashref'=> $Response}]
    );
    return $Mockify->getMockObject();
}

sub _createShopUriBuilder {
    my $self = shift;
    my ($Shop, $System) = @_;

    my $aParameterList = [$Shop, $System];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::Shop::API::ShopUriBuilder',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('buildPrivateUri', 'http://baseuri.com', [
        {'string' => 'accounting/sageone/{shopGuid}/authurl'},
        {'hashref' => {'shopGuid'=>'MyShopAlias'}},
        {'hashref' => {
            'ep6_redirect_url'=>'http://epages6.com/url',
            'public_epj_redirect_url' => 'http://epagesj.com/url'
        }}
    ]);

    return $Mockify->getMockObject();
}
__PACKAGE__->runScript();