#========================================================================================
# package      DE_EPAGES::SageOne::t::unit::ShopService_Sync
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::SageOne::API::Webservice::ShopService
#========================================================================================
package DE_EPAGES::SageOne::t::unit::ShopService_Sync;
use base DE_EPAGES::Test::API::UnitTest::Test;
use DE_EPAGES::Test::Mock::EpagesObjectMockFactory;
use DE_EPAGES::Test::Mock::Mockify qw (WasCalled);
use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::SageOne::API::Constants qw ( SAGE_SHOP_SYNC_PATH_TEMPLATE );
use Test::More;
use Test::Exception;

use strict;

sub test {
    my $self = shift;

    $self->_buildMocks();

    $self->test_DoRequest_Successful();
    $self->test_DoRequest_WebserviceError();
    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_DoRequest_Successful {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory()->get('SUT_Successful');

    $SUT->syncShop();

    my $wasCalled = WasCalled($SUT, 'sendGetRequest');
    is($wasCalled, 1, "$SubTestName - test if sendGetRequest was called");
}

sub test_DoRequest_WebserviceError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory()->get('SUT_WebserviceError');

    throws_ok(sub { $SUT->syncShop(); }, qr/WebserviceError/, "$SubTestName - test if syncShop throws WebserviceError");
}

#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _buildMocks {
    my $self = shift;
    my $Shop = $self->mockFactory()->buildShop({
        'GUID'=>'MyShopAlias',
        'RESTAPIURL' => 'http://epagesj/base/url/',
        })->getMockObject();

    my $SUT_Successful = $self->_createSUTSuccessful($Shop);
    my $SUT_WebserviceError = $self->_createSUTWebserviceError($Shop);

    $self->memory()->add('SUT_Successful', $SUT_Successful);
    $self->memory()->add('SUT_WebserviceError', $SUT_WebserviceError);
}
#----------------------------------------------------------------------------------------
sub _createSUTSuccessful {
    my $self = shift;
    my ($Shop) = @_;

    my $aParameterList = [$Shop];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::ShopService',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('_createShopUriBuilder', $self->_createShopUriBuilder($Shop), [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]);
    $Mockify->addMockWithReturnValueAndParameterCheck('sendGetRequest', undef, [{'string' => 'http://baseuri.com'}, {'hashref'=> {}}]
    );

    return $Mockify->getMockObject();
}

sub _createSUTWebserviceError {
    my $self = shift;
    my ($Shop) = @_;

    my $aParameterList = [$Shop];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::ShopService',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('_createShopUriBuilder', $self->_createShopUriBuilder($Shop), [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]);
    $Mockify->addMock('sendGetRequest', sub{ Error('WebserviceError');});

    return $Mockify->getMockObject();
}

sub _createShopUriBuilder {
    my $self = shift;
    my ($Shop) = @_;

    my $aParameterList = [$Shop, $self->_createSystem()];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::Shop::API::ShopUriBuilder',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('buildPrivateUri', 'http://baseuri.com', [
        {'string' => SAGE_SHOP_SYNC_PATH_TEMPLATE},
        {'hashref' => {'shopGuid'=>'MyShopAlias'}}
    ]);

    return $Mockify->getMockObject();
}

sub _createSystem {
    my $self = shift;
    my $hAttributes = {
        'EPagesJBaseURL' => 'http://baseurl.com',
    };
    return $self->mockFactory()->buildSite($hAttributes)->getMockObject();
}

__PACKAGE__->runScript();