﻿package DE_EPAGES::TaxUS::t::unit::TaxMatrix;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::TaxUS::Hooks::TaxMatrix;
use DE_EPAGES::Test::Mock::EpagesObjectMockFactory;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::More tests => 8;

use strict;
use utf8;

sub test {
    my $self = shift;

    $self->test_OnDisplayTaxAreaAndRate_UsShop_UsAddress();
    $self->test_OnDisplayTaxAreaAndRate_UsShop_NonUsAddress();
    $self->test_OnDisplayTaxAreaAndRate_UsShop_UsUser();
    $self->test_OnDisplayTaxAreaAndRate_UsShop_NonUsUser();
    $self->test_OnDisplayTaxAreaAndRate_NonUsShop_UsAddress();
    $self->test_OnDisplayTaxAreaAndRate_NonUsShop_NonUsAddress();
    $self->test_OnDisplayTaxAreaAndRate_NonUsShop_UsUser();
    $self->test_OnDisplayTaxAreaAndRate_NonUsShop_NonUsUser();

    return;
}

sub test_OnDisplayTaxAreaAndRate_UsShop_UsAddress {
    my $self = shift;

    my $Shop           = $self->_createUsShop();
    my $BillingAddress = $self->_createAddress( $self->_usCountryID );
    my $Basket         = $self->_createBasket( $Shop, $BillingAddress );

    my $hParam = {
        'BasketOrOrder' => $Basket,
        'displayed'     => 1,
    };

    DE_EPAGES::TaxUS::Hooks::TaxMatrix::OnDisplayTaxAreaAndRate($hParam);

    is( $hParam->{'displayed'}, 0, 'OnDisplayTaxAreaAndRate US shop US address' );

    return;
}

sub test_OnDisplayTaxAreaAndRate_UsShop_NonUsAddress {
    my $self = shift;

    my $Shop           = $self->_createUsShop();
    my $BillingAddress = $self->_createAddress( $self->_nonUsCountryID );
    my $Basket         = $self->_createBasket( $Shop, $BillingAddress );

    my $hParam = {
        'BasketOrOrder' => $Basket,
        'displayed'     => 1,
    };

    DE_EPAGES::TaxUS::Hooks::TaxMatrix::OnDisplayTaxAreaAndRate($hParam);

    is( $hParam->{'displayed'}, 1, 'OnDisplayTaxAreaAndRate US shop non US address' );

    return;
}

sub test_OnDisplayTaxAreaAndRate_UsShop_UsUser  {
    my $self = shift;

    my $Shop           = $self->_createUsShop();
    my $BillingAddress = $self->_createAddress( $self->_usCountryID );
    my $User           = $self->_createUser($BillingAddress);
    my $Basket         = $self->_createBasket( $Shop, undef, undef, $User );

    my $hParam = {
        'BasketOrOrder' => $Basket,
        'displayed'     => 1,
    };

    DE_EPAGES::TaxUS::Hooks::TaxMatrix::OnDisplayTaxAreaAndRate($hParam);

    is( $hParam->{'displayed'}, 0, 'OnDisplayTaxAreaAndRate US shop US customer' );

    return;
}

sub test_OnDisplayTaxAreaAndRate_UsShop_NonUsUser {
    my $self = shift;

    my $Shop           = $self->_createUsShop();
    my $BillingAddress = $self->_createAddress( $self->_nonUsCountryID );
    my $User           = $self->_createUser($BillingAddress);
    my $Basket         = $self->_createBasket( $Shop, undef, undef, $User );

    my $hParam = {
        'BasketOrOrder' => $Basket,
        'displayed'     => 1,
    };

    DE_EPAGES::TaxUS::Hooks::TaxMatrix::OnDisplayTaxAreaAndRate($hParam);

    is( $hParam->{'displayed'}, 1, 'OnDisplayTaxAreaAndRate US shop non US customer' );

    return;
}

sub test_OnDisplayTaxAreaAndRate_NonUsShop_UsAddress {
    my $self = shift;

    my $Shop            = $self->_createNonUsShop();
    my $BillingAddress  = $self->_createAddress( $self->_usCountryID );
    my $Basket          = $self->_createBasket( $Shop, $BillingAddress );

    my $hParam = {
        'BasketOrOrder' => $Basket,
        'displayed'     => 1,
    };

    DE_EPAGES::TaxUS::Hooks::TaxMatrix::OnDisplayTaxAreaAndRate($hParam);

    is( $hParam->{'displayed'}, 1, 'OnDisplayTaxAreaAndRate non US shop US address' );

    return;
}

sub test_OnDisplayTaxAreaAndRate_NonUsShop_NonUsAddress {
    my $self = shift;

    my $Shop            = $self->_createNonUsShop();
    my $BillingAddress  = $self->_createAddress( $self->_nonUsCountryID );
    my $Basket          = $self->_createBasket( $Shop, $BillingAddress );

    my $hParam = {
        'BasketOrOrder' => $Basket,
        'displayed'     => 1,
    };

    DE_EPAGES::TaxUS::Hooks::TaxMatrix::OnDisplayTaxAreaAndRate($hParam);

    is( $hParam->{'displayed'}, 1, 'OnDisplayTaxAreaAndRate non US shop non US address' );

    return;
}

sub test_OnDisplayTaxAreaAndRate_NonUsShop_UsUser {
    my $self = shift;

    my $Shop           = $self->_createNonUsShop();
    my $BillingAddress = $self->_createAddress( $self->_usCountryID );
    my $User           = $self->_createUser($BillingAddress);
    my $Basket         = $self->_createBasket( $Shop, undef, undef, $User );

    my $hParam = {
        'BasketOrOrder' => $Basket,
        'displayed'     => 1,
    };

    DE_EPAGES::TaxUS::Hooks::TaxMatrix::OnDisplayTaxAreaAndRate($hParam);

    is( $hParam->{'displayed'}, 1, 'OnDisplayTaxAreaAndRate non US shop US customer' );

    return;
}

sub test_OnDisplayTaxAreaAndRate_NonUsShop_NonUsUser {
    my $self = shift;

    my $Shop           = $self->_createNonUsShop();
    my $BillingAddress = $self->_createAddress( $self->_nonUsCountryID );
    my $User           = $self->_createUser($BillingAddress);
    my $Basket         = $self->_createBasket( $Shop, undef, undef, $User );

    my $hParam = {
        'BasketOrOrder' => $Basket,
        'displayed'     => 1,
    };

    DE_EPAGES::TaxUS::Hooks::TaxMatrix::OnDisplayTaxAreaAndRate($hParam);

    is( $hParam->{'displayed'}, 1, 'OnDisplayTaxAreaAndRate non US shop non US customer' );

    return;
}

sub _createBasket {
    my $self = shift;
    my ( $Shop, $BillingAddress, $ShippingAddress, $User ) = @_;

    return DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::Basket',
        {
            'ID'              => 12345,
            'Site'            => $Shop,
            'BillingAddress'  => $BillingAddress,
            'ShippingAddress' => $ShippingAddress,
            'User'            => $User,
        }
    )->getMockObject();
}

sub _createUsShop {
    my $self = shift;

    return $self->_createShop( 840, 'TaxMatrixUS' );
}

sub _createNonUsShop {
    my $self = shift;

    return $self->_createShop( 100, 'TaxMatrixDE' );
}

sub _createShop {
    my $self = shift;
    my ( $CountryID, $TaxClassAlias ) = @_;

    return $self->mockFactory->buildShop(
        {   'Address'   => $self->_createAddress($CountryID),
            'TaxMatrix' => $self->_createTaxMatrix($TaxClassAlias),
            'Parent'    => $self->_createShopsFolder(),

        }
    )->getMockObject();
}

sub _createAddress {
    my $self = shift;
    my ($CountryID) = @_;

    return DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Object::API::Object::Object',
        { 'CountryID' => $CountryID } )->getMockObject();
}

sub _createTaxMatrix {
    my $self = shift;
    my ($TaxClassAlias) = @_;

    return DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Tax::API::Object::TaxMatrix',
        {
            'Class' => DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
                'DE_EPAGES::Object::API::Object::Class',
                { 'Alias' => $TaxClassAlias }
            )->getMockObject()
        }
    )->getMockObject();
}

sub _usCountryID {
    my $self = shift;

    return 840;
}

sub _nonUsCountryID {
    my $self = shift;

    return 100;
}

sub _createShopsFolder {
    my $self = shift;

    return DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Object::API::Object::Object',
        {
            'Site' => $self->_createSystem()
        }
    )->getMockObject();
}

sub _createSystem {
    my $self = shift;

    return DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Object::API::Object::Object',
        {
            'StoreName' => 'Store'
        }
    )->getMockObject();
}

sub _createUser {
    my $self = shift;
    my ($BillingAddress) = @_;

    return DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Permission::API::Object::User',
        {
            'BillingAddress' => $BillingAddress
        }
    )->getMockObject();
}

__PACKAGE__->runScript;
1;
