#========================================================================================
# package      DE_EPAGES::WebService::Samples::Server::HelloResource
# state        private
# base         DE_EPAGES::WebService::API::Resource::BaseResource
#----------------------------------------------------------------------------------------
# description  Very basic example for an implementation of a REST resource.
#               Implements the methods GET, PUT, DELETE and POST.
#========================================================================================
package DE_EPAGES::WebService::Samples::Server::HelloResource;
use base DE_EPAGES::WebService::API::Resource::BaseResource;

use strict;
use DE_EPAGES::Core::API::Error qw (Error);

use constant MAX_RAND => 100000;

my %Items; # in-memory-storage

sub get {
    my $self = shift;
    my ($ID) = @_;

    Error( 'ResourceNotFound', { ID => $ID } ) unless exists $Items{$ID};
    return $Items{$ID};
}

sub put {
    my $self = shift;
    my ($ID, $Data) = @_;

    $Items{$ID} = $Data;
    return;
}

sub delete {
    my $self = shift;
    my ($ID) = @_;

    delete $Items{$ID};
    return;
}

sub post {
    my $self = shift;
    my (undef, $Data) = @_;

    my $ID = int(rand(MAX_RAND)); # always generate a new id
    $ID = int(rand(MAX_RAND)) while exists $Items{$ID};
    $Items{$ID} = $Data;
    return { id => $ID };
}

sub match {
    my $self = shift;
    my ($Request) = @_;

    return unless $Request->uri->path =~ m#^/api/hello/?$|^/api/hello/([^/]*)/?$#;

    my $Match = $1 // '';
    return $Match;
}

1;
